package malaria

String writeDir
if (args.size()!=1) {
    println "Argument: persistence_dir"
    System.exit(-1)
}
else {
    writeDir = args[0]
}

int GENS = 1000
double START_FREQ = 0.003D
double THRESHOLD  = 0.1D
List<Map<Integer,Double>> workMults = [TypicalScenarios.twoClonal]
List<Double> resLevels = [0D]
List<Double> TSs       = [0D]
List<Double> Ss        = TypicalScenarios.massiveS
List<Double> dFreqs    = TypicalScenarios.massiveDFreqs
boolean isSI           = false
int numDrugs           = 2
int locPerDrug         = 1
int minInterval        = 0

double getSumMut(double[] freqs, int mut) {
    double sum = 0
    //println mut + "---"
    for(int i=0; i<freqs.length; i++) {
            if ((i & (int)Math.pow(2,mut)) != 0) {
                //println "i " + i
                sum += freqs[i]
            }
    }
    return sum
}

Map<Double,Double> getAverageS(Simulation sim,
    double minDF, double maxDF, int locPos) {
    //Getting average s of mutation of drug 1 after drug2 intro
    Simulation mySim = sim.getSimDrugFreq(minDF, maxDF)
    Map<Double,List<Double>> grabber = [:]
    sim.results.each {
        ResultClassifier rc = it.key
        PolicyResult result = it.value
        double[][] genFreqs = result.genFreqs
        if ((result.swapTimes != null) &&
            (result.swapTimes.length>1)) {
            int startPoint = result.swapTimes[1]
            int endPoint = result.genFreqs.length
            if (result.swapTimes.length>2) {
                endPoint = result.swapTimes[2]
            }
            double oldFreq = getSumMut(genFreqs[startPoint], locPos)
            int numPoints = 0
            double addS = 0
            //println "df: " + rc.dFreq
            //println startPoint + " " + endPoint + " " + result.swapTimes
            //for (int gen=0;gen<result.genFreqs.length; gen++) {
            //    println "gg: " + gen + " " + getSumMut(genFreqs[gen], locPos)
            //}
            for (int gen=startPoint+1;gen<endPoint; gen++) {
                double newFreq = getSumMut(genFreqs[gen], locPos)
                //println gen +  " " + oldFreq + " " + newFreq
                addS += newFreq/oldFreq
                numPoints++
                oldFreq = newFreq
            }
            //println "bb " + addS + " " + numPoints
            if (numPoints>0) {
                grabber[rc.s] = (grabber[rc.s] ?: []) + (addS/numPoints)
            }
            //println addS/numPoints
        }
    }
    Map<Double, Double> agg = [:]
    grabber.each {
        double df = it.key
        List<Double> Ss = it.value
        double sum = 0
        for (s in Ss) {
            sum +=s
        }
        agg[df] = sum/(Ss.size())
    }
    return agg
}


Map<Double,Double> actS
Map<Double,Double> standardS

Persistence writer = new Persistence(writeDir)
Simulation sim = new Simulation(GENS, writer, 0, START_FREQ, THRESHOLD, false)
SimulationRunner sr = new SimulationRunner(sim,
    workMults, resLevels, TSs, Ss, dFreqs, isSI)
sr.runACT(numDrugs, minInterval)

actS = getAverageS(sim, 0.1D, 0.8D, 1) // 1 = non art of drug 1

sim = new Simulation(GENS, writer, 0, START_FREQ, THRESHOLD, false)
sr = new SimulationRunner(sim, workMults, resLevels, TSs, Ss, dFreqs, isSI)
sr.runStandard(numDrugs, locPerDrug, minInterval)
standardS = getAverageS(sim, 0.1D, 0.8D, 0) // 0 = drug 1

fName = "ACTd1at2.png"
chart = Chart.plotSCompare(actS, standardS)
out = new FileOutputStream(fName)
Chart.saveChart(out, chart)
out.close()
