package malaria;

/**
 * Analysis class.
 *
 * @author tra
 */

import java.util.*;

public class Analysis {

    /* Requires all mutPos BUT
     *   if mutPos = [] must be sensitive
     */
    public static double[] getEffectiveSForFreq(PolicyResult result, int[] mutPos) {
        double[] fs = new double[result.genFreqs.length - 1];
        for (int i = 1; i < result.genFreqs.length; i++) {
            double[] freqsMinus1 = result.genFreqs[i - 1];
            double[] freqs = result.genFreqs[i];
            double sumMinus1 = 0;
            double sum = 0;
            for (int hapl=0; hapl<freqs.length; hapl++) {
                boolean useable = true;
                if (mutPos.length == 0) {
                    useable = (hapl==0);
                }
                else {
                    for (int mut : mutPos) {
                        if ((hapl & (int)Math.pow(2, mut)) == 0) {
                            useable = false;
                            break;
                        }
                    }
                }
                if (useable) {
                    sumMinus1 += freqsMinus1[hapl]; 
                    sum += freqs[hapl]; 
                    //System.out.println("Analyisis: " + hapl + " " + sum + " " + sumMinus1 + " " + sum/sumMinus1);
                }
            }
            fs[i - 1] = sum / sumMinus1;
            if ((i>1) && (fs[i-2]==Double.POSITIVE_INFINITY)) {
                fs[i-2]= fs[i-1]; //Just to correct possible initial values
            }
        //System.out.println("analysis " + fs[0][i-1] + " " + fs[1][i-1]);
        }
        return fs;
    }

    public static double getAvgR(PolicyResult result, int[] mutPos, int limit) {
        double sumR = 0;
        double[] r;
        r = getEffectiveSForFreq(result, mutPos);
	//Ignoring 1st gen
        for(int i=1;i<limit; i++) {
	    System.out.println("XXX  " + r[i]);
            sumR += r[i];
        }
        return sumR/limit;

    }

    static double getAllResistants(int[] muts, double[] freqs) { //must change to int[] mut
        double resistance = 0;
        for (int i = 0; i < freqs.length; i++) {
            boolean resistant = true;
            for (int mut: muts) {
                if ((i & (int) Math.pow(2, mut)) == 0) {
                    resistant = false;
                    break;
                }
            }
            if (resistant) resistance += freqs[i];
        }
        return resistance;
    }
    
    static double getAllSensitives(int[][] drugMuts, double[] freqs) {
        boolean noRes = true;
        double sensitivity = 0;
        for (int i = 0; i < freqs.length; i++) {
            boolean sensitive = true;
            for (int[] muts : drugMuts) {
                boolean drugResistant = true;
                for (int mut : muts) {
                    if ((i & (int) Math.pow(2, mut)) == 0) {
                        drugResistant = false;
                        break;
                    }
                }
                if (drugResistant) {
                    sensitive = false;
                    break;
                }
            }
            if (sensitive) sensitivity += freqs[i];
        }
        return sensitivity;
    }
       
    public static int gensToThr(PolicyResult result, int[][] drugMuts, double thres) {
    //int[] drugMuts must be int[][] drugMuts
        for (int currGen = 0; currGen < result.genFreqs.length; currGen++) {
            for (int[] muts : drugMuts) {
                if (Analysis.getAllResistants(muts, result.genFreqs[currGen]) >= thres) {
                    return currGen;
                }
            }
        }
        return -1;
    }

    //Assumes 2 loci
    public static double[] reportLD(PolicyResult result, int[] left, int[] right) {
        double[] ld = new double[result.genFreqs.length];
        for (int gen=0; gen<result.genFreqs.length; gen++) {
            try {
                LD ldCalc = new LD(result.genFreqs[gen], left, right);
                ld[gen] = ldCalc.getLD();
            }
            catch (ArrayIndexOutOfBoundsException ae) {
                ae.printStackTrace();
                System.out.println("\nBAT"+gen +" "+ result.genFreqs[gen].length);
                System.exit(-1);
            }
        }
        return ld;
    }

    //Assures that a swap lasts longer than numGens gens
    //returns generatation where it stops doing that
    static int assureSwap(PolicyResult result, int numGens) {
        if (result.swapTimes == null || result.swapTimes.length == 1) {
            return result.genFreqs.length;
        }
        for (int i=1;i<result.swapTimes.length;i++) {
            if (result.swapTimes[i-1]+numGens>=result.swapTimes[i]) {
                return result.swapTimes[i-1];
            }
        }
        return result.genFreqs.length;
    }
    
    public static int reportDurationStandard(int gens,
            ResultClassifier cls, PolicyResult result,
            int[][] drugMuts, double thres, int epiType) {
        int duration;
        if (cls.getPolicy().equals("ThresRot")) {
            //duration = Analysis.reportDurationThrRot(gens, it.value,
            duration = Analysis.reportDurationSum(gens, result,
                drugMuts, thres, epiType);
            //if (duration == 0) {
            //    println "Table malaria " + it.key + " " + it.value
            //}
        }
        else if (cls.getPolicy().equals("ThresRotFixed")) {
            System.out.println("report probably needs epiType: Check");
            duration = Analysis.reportDurationThrRotFixed(gens, result,
                drugMuts, thres);
        }
        else if (cls.getPolicy().equals("MFT")) {
            duration = Analysis.reportDurationSum(gens, result,
                drugMuts, thres, epiType);
        }
        else if (cls.getPolicy().equals("CT")) {
            int[][] myDrugMuts = new int[1][drugMuts.length*drugMuts[0].length];
            for (int d=0; d<drugMuts.length; d++) {
                for (int pos=0; pos<drugMuts[d].length; pos++) {
                    myDrugMuts[0][d*drugMuts[0].length + pos] = drugMuts[d][pos];
                }
            }
            duration = Analysis.reportDurationSum(gens, result,
                myDrugMuts, thres, epiType);
        }
        else { //We dont really know
            duration = Analysis.reportDurationSum(gens, result,
                drugMuts, thres, epiType);
        }
        return duration;
    }

    public static int reportDurationSingle(int gens,
            PolicyResult result, int[][] drugMuts, double thres) {
        int numDrugs = drugMuts.length;
        //System.out.println("Analisys: " + gens + " " + result.genFreqs.length);
        for (int currGen = result.genFreqs.length - 1; currGen >= 0; currGen--) {
            double[] resist = new double[numDrugs];
            double sumResist = 0;
            double maxResist = 0;
            double minResist = 1;

            for (int di=0; di<numDrugs; di++) {
                int[] muts = drugMuts[di];
                resist[di] = Analysis.getAllResistants(muts, result.genFreqs[currGen]);
                if (maxResist<resist[di]) {
                    maxResist = resist[di];
                }
                if (minResist>resist[di]) {
                    minResist = resist[di];
                }
                sumResist += resist[di];
            }
            if (currGen == result.genFreqs.length-1) {
                if (result.genFreqs[currGen][0]>0.99) {
                    //sensitive dominates even at end
                    return -1;
                }
                else if (maxResist<thres) {
                    return -gens; //unknown  (fixation or needed more gens)
                }
            }
            if (maxResist<thres) return currGen;
            //stop when resistance is below thres
            //(unless the sum of all res is bigger than numDrugs*Thres
        }
        //if (result.genFreqs.length<gens) return -1; //assuming 0 is sensible
        if (result.swapTimes != null) {
            return result.swapTimes[result.swapTimes.length-1];
        }
        else {
            return 0;
        }
    }

    public static int reportDurationAll(int gens,
            PolicyResult result, int[][] drugMuts, double thres) {
        int numDrugs = drugMuts.length;
        //System.out.println("Analisys: " + gens + " " + result.genFreqs.length);
        for (int currGen = result.genFreqs.length - 1; currGen >= 0; currGen--) {
            double[] resist = new double[numDrugs];
            double sumResist = 0;
            double maxResist = 0;
            double minResist = 1;

            for (int di=0; di<numDrugs; di++) {
                int[] muts = drugMuts[di];
                resist[di] = Analysis.getAllResistants(muts, result.genFreqs[currGen]);
                if (maxResist<resist[di]) {
                    maxResist = resist[di];
                }
                if (minResist>resist[di]) {
                    minResist = resist[di];
                }
                sumResist += resist[di];
            }
            if (currGen == result.genFreqs.length-1) {
                if (result.genFreqs[currGen][0]>0.99) {
                    //sensitive dominates even at end
                    return -1;
                }
                else if (minResist<thres) {
                    return -gens; //unknown  (fixation or needed more gens)
                }
            }
            if (minResist<thres) return currGen;
            //stop when resistance is below thres
            //(unless the sum of all res is bigger than numDrugs*Thres
        }
        //if (result.genFreqs.length<gens) return -1; //assuming 0 is sensible
        return 0;
    }

    public static int reportDurationSum(int gens,
            PolicyResult result, int[][] drugMuts, double thres,
            int epiType) {
        int numDrugs = drugMuts.length;
        int numClones = result.genFreqs[0].length;
        //System.out.println("Analisys: " + gens + " " + result.genFreqs.length);
        for (int currGen = result.genFreqs.length - 1; currGen > 0; currGen--) {
            double[] resist = new double[numDrugs];
            double sumResist = 0;
            double maxResist = 0;
            double minResist = 1;
            HashSet<Integer> setCl = new HashSet<Integer>();
            
            for (int di=0; di<numDrugs; di++) {
                int[] muts = drugMuts[di];
                resist[di] = Analysis.getAllResistants(muts, result.genFreqs[currGen]);
                if (maxResist<resist[di]) {
                    maxResist = resist[di];
                }
                if (minResist>resist[di]) {
                    minResist = resist[di];
                }
                for (int cl=0; cl<numClones; cl++) {
                    boolean isResistant;
                    if (epiType != 1) {
                        isResistant = true;
                        for (int mut : muts) {
                            if ((cl & (int)Math.pow(2, mut)) == 0) {
                                isResistant = false;
                                break;
                            }
                        }
                    }
                    else {
                        isResistant = false;
                        for (int mut : muts) {
                            if ((cl & (int)Math.pow(2, mut)) == 1) {
                                isResistant = true;
                                break;
                            }
                        }
                    }
                    if (isResistant) setCl.add(cl);
                }
            }
            for (int cl=0; cl<numClones; cl++) {
                if (setCl.contains(cl)) sumResist += result.genFreqs[currGen][cl];
            }
            if (currGen == result.genFreqs.length-1) {
                if (result.genFreqs[currGen][0]>0.99) {
                    //sensitive dominates even at end
                    return -1;
                }
                else if (sumResist<thres) {
                    return -gens; //unknown  (fixation or needed more gens)
                }
            }
            if (sumResist<thres) return currGen;
            //stop when resistance is below thres
            //(unless the sum of all res is bigger than numDrugs*Thres
        }
        //if (result.genFreqs.length<gens) return -1; //assuming 0 is sensible
        if (result.swapTimes != null && result.swapTimes.length>1) {
            return 1;
            //System.out.println("BLA" + result.swapTimes[result.swapTimes.length-1]
            //        + " " + result.swapTimes.length);
            //return result.swapTimes[result.swapTimes.length-1];
        }
        else {
            return 1;
        }
    }
    
    
    public static int reportDurationMulti(int gens,
            PolicyResult result, int[][] drugMuts, double thres) {
        int numDrugs = drugMuts.length;
        //System.out.println("Analisys: " + gens + " " + result.genFreqs.length);
        for (int currGen = result.genFreqs.length - 1; currGen >= 0; currGen--) {
            double multiResistance;

            multiResistance = result.genFreqs[currGen][result.genFreqs[currGen].length-1];
            if (currGen == result.genFreqs.length-1) {
                if (result.genFreqs[currGen][0]>0.99) {
                    //sensitive dominates even at end
                    return -1;
                }
                else if (multiResistance<thres) {
                    return -gens; //unknown  (fixation or needed more gens)
                }
            }

            if(multiResistance<thres) return currGen;
        }
        //if (result.genFreqs.length<gens) return -1; //assuming 0 is sensible
        return 0;
    }


    public static int reportDurationThrRot(int gens,
        PolicyResult result, int[][] drugMuts, double thres) {
        int numDrugs = drugMuts.length;
        int maxSwap = assureSwap(result, 5);
        if (result.genFreqs[result.genFreqs.length-1][0]>0.99) {
            return -1;
        }
        if (maxSwap==result.genFreqs.length) return -gens;
            return maxSwap;
    }

        
    public static int reportDurationThrRotFixed(int gens,
            PolicyResult result, int[][] drugMuts, double thres) {
        int numDrugs = drugMuts.length;
        for (int currGen = result.genFreqs.length - 1; currGen >= 0; currGen--) {
            double sensitivity;

            sensitivity = Analysis.getAllSensitives(drugMuts, result.genFreqs[currGen]);

            if (currGen == result.genFreqs.length-1) {
                if (result.genFreqs[currGen][0]>0.99) {
                    //sensitive dominates even at end
                    return -1;
                }
                else if (sensitivity>1-numDrugs*thres) {
                    return -gens; //unknown  (fixation or needed more gens)
                }
            }
            if (sensitivity>1-numDrugs*thres) return currGen;                
        }
        //if (result.genFreqs.length<gens) return -1; //assuming 0 is sensible
        return 0;
    }

    
    
}
