package malaria

import javax.swing.JTextPane
import javax.swing.JPanel
import org.jfree.chart.ChartFactory
import org.jfree.chart.JFreeChart
import org.jfree.data.xy.XYDataset
import org.jfree.chart.plot.XYPlot
import org.jfree.chart.ChartPanel
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer
import org.jfree.chart.plot.PlotOrientation
import org.jfree.data.xy.XYSeriesCollection
import org.jfree.data.xy.XYSeries
import java.awt.Color

/**
 *
 * @author tiago
 */
class ChartPane {
    ChartPanel cPanel
    XYSeriesCollection dataset

    public ChartPane() {
        dataset = new XYSeriesCollection()
        JFreeChart chart = createChart(dataset)
        cPanel = new ChartPanel(chart)
        cPanel.setPreferredSize(new java.awt.Dimension(520, 300));
        
    }

    public getPane() {
        return cPanel
    }

    public inform(s, dFreq, PolicyResult pr) {
        double[][] genFreqs = pr.genFreqs
        dataset.removeAllSeries()
        int numSeries = genFreqs[0].length
        for (int sI=0; sI<numSeries; sI++) {
            XYSeries serie = new XYSeries(sI.toString())
            for (int gen=0; gen<genFreqs.length; gen++) {
                serie.add(gen, genFreqs[gen][sI])
            }
            dataset.addSeries(serie)
        }
        XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) cPanel.getChart().getPlot().getRenderer()
        for (int sI=0; sI<numSeries; sI++) {
            renderer.setSeriesLinesVisible(sI, true)
            renderer.setSeriesShapesVisible(sI, false)
            println 255*((sI+1)/numSeries)
            renderer.setSeriesPaint(sI, new Color((int)255*((sI+1)/numSeries),0,0))
        }
        double myS = Math.round(s*1000)/1000
        double myDFreq = Math.round(dFreq*1000)/1000
        cPanel.getChart().setTitle("s: ${myS}, drug: ${myDFreq}")
        cPanel.paintImmediately(0,0,cPanel.getWidth(),cPanel.getHeight())
    }

    private JFreeChart createChart(XYDataset dataset) {
        // create the chart...
        JFreeChart chart = ChartFactory.createXYLineChart(
            "Frequency over time",      // chart title
            "Time",                      // x axis label
            "Frequency",                      // y axis label
            dataset,                  // data
            PlotOrientation.VERTICAL,
            false,                     // include legend
            false,                     // tooltips
            false                     // urls
        );

        XYPlot plot = (XYPlot) chart.getPlot();
        plot.getDomainAxis().setLowerMargin(0.0);
        plot.getDomainAxis().setUpperMargin(0.0);

        return chart;
    }


}

