package malaria;

import java.math.*;

public class LD {
    double[][] haploFreqs;
    double ld;

    public LD(double[][] haploFreqs) {
        this.haploFreqs = haploFreqs.clone();
        calcLD();

    }

    //binary loci
    public LD(double[] genome, int[] left, int[] right) {
        this.haploFreqs = new double[2][2];
        for(int pos=0; pos<genome.length; pos++) {
            int p1=1, p2=1;
            for (int locus : left) {
                if ((pos & (int)Math.pow(2, locus))==0) {
                    p1 = 0;
                    break;
                }
            }
            for (int locus : right){
                if ((pos & (int)Math.pow(2, locus))==0) {
                    p2 = 0;
                    break;
                }
            }
            //System.out.println ("pos:" + pos + " p1:" + p1 + " p2:" + p2 );
            haploFreqs[p1][p2] += genome[pos];
        }
        calcLD();
    }

    void calcLD() {
        calcLDr();
    }

    void calcLDDLine() {
        double[] partialA = new double[haploFreqs.length];
        double[] partialB = new double[haploFreqs[0].length];
        //BigDecimal D;
        double D;

        for(int a=0; a<partialA.length; a++) {
            for(int b=0; b<partialB.length; b++) {
                partialA[a] += haploFreqs[a][b];
                partialB[b] += haploFreqs[a][b];
            }
        }
        D = haploFreqs[0][0] - partialA[0]*partialB[0];
        //D = BigDecimal.valueOf(haploFreqs[0][0]).subtract(
        //        BigDecimal.valueOf(partialA[0]).multiply(
        //        BigDecimal.valueOf(partialB[0])));
        //BigDecimal div, div2;
        double div, div2;
        //if (D.compareTo(BigDecimal.ZERO)>0) {
        if (D>0) {
            div  = partialA[0]*partialB[1];
            div2 = partialA[1]*partialB[0];
            if (div<div2) div = div2;
            //div = BigDecimal.valueOf(partialA[0]).multiply(
            //        BigDecimal.valueOf(partialB[1]));
            //div2 = BigDecimal.valueOf(partialA[1]).multiply(
            //        BigDecimal.valueOf(partialB[0]));
            //if (div.compareTo(div2)<0) div = div2;
        }
        else {
            div  = partialA[0]*partialB[0];
            div2 = partialA[1]*partialB[1];
            if (div>div2) div = div2;
            //div = BigDecimal.valueOf(partialA[0]).multiply(
            //        BigDecimal.valueOf(partialB[0]));
            //div2 = BigDecimal.valueOf(partialA[1]).multiply(
            //        BigDecimal.valueOf(partialB[1]));
            //if (div.compareTo(div2)>0) div = div2;
            //div = -div;
        }
        if (div==0) {
        //if (div.compareTo(BigDecimal.ZERO)==0) {
            ld=0;
        }
        else {
            ld = D / div;
            //ld = D.divide(div, RoundingMode.HALF_EVEN).doubleValue();
        }
        //if (ld<-1) {
        //    System.out.println("Ld is problematic " + ld + " " + D + " " + div);
            //System.out.println(haploFreqs[0][0]);
            //System.out.println(haploFreqs[0][1]);
            //System.out.println(haploFreqs[1][0]);
            //System.out.println(haploFreqs[1][1]);

        //}
    }

    void calcLDr() {
        double[] partialA = new double[haploFreqs.length];
        double[] partialB = new double[haploFreqs[0].length];
        double D;

        for(int a=0; a<partialA.length; a++) {
            for(int b=0; b<partialB.length; b++) {
                partialA[a] += haploFreqs[a][b];
                partialB[b] += haploFreqs[a][b];
            }
        }
        D = haploFreqs[0][0] - partialA[0]*partialB[0];
        double div=1;
        for (int a=0;a<partialA.length; a++) {
            div *= partialA[a];
        }
        for (int b=0; b<partialB.length; b++) {
            div *= partialB[b];
        }
        div = Math.sqrt(div);
        if (div==0) {
              ld=0;
        }
        else {
            ld = D / div;
        }
        //if (ld<-1) {
        //    System.out.println("Ld is problematic " + ld + " " + D + " " + div);
        //}
    }



    public double getLD() {
        return ld;
    }

    public static void main(String[] args) {
        double[] genome = {
            0.125, 0.125, 0.125, 0.125,
            0.125, 0.125, 0.125, 0.125};
        int[] left = {0,1};
        int[] right = {0,2};
        LD ld = new LD(genome, left, right);
        System.out.println(ld.getLD());
    }
}
