package malaria

/**
 *
 * @author tra
 */


class NoChart {
    static dumpList(File f, List l) {
        for (i in l) {
            f << i + " "
        }
        f << "\n"
    }

    static void plotSwaps(PolicyResult result, String dir, String fName) {
        //Not Credit Default Swaps ;)
        if (result.usedDrugs == null) {
            return
        }
        File oF = new File(dir + File.separator + fName)
        oF.delete()
        for (int i=0; i<result.usedDrugs.length; i++) {
            oF << "${result.swapTimes[i]} ${result.usedDrugs[i]}\n"
        }
    }

    static void plotLD(PolicyResult result, String dir, String fName) {
        //Just for 1 drug/2 loci
        File oF = new File(dir + File.separator + fName)
        oF.delete()
        result.genFreqs.eachWithIndex { gens, g ->
            LD ldC = new LD(gens, (int[])[0], (int[])[1])
            double ld = ldC.getLD()
            oF << "${ld} "
        }
        oF << "\n"
    }



    static void plotGenepop(PolicyResult result,
        int numMuts, int epiType, String dir, String fName,
        int sampSize) {
        File oF = new File(dir + File.separator + fName)
        oF.delete()
        oF << "ogaraK generated file, circa ${sampSize}/generation\n"
        for (int currLoc=0;currLoc<numMuts;currLoc++) {
            oF << "lc${currLoc+1}\n"
        }
        int currPop = 1
        result.genFreqs.eachWithIndex { gens, g ->
            oF << "Pop\n"
            int currInd = 1
            gens.eachWithIndex {freq, locus ->
                int numReps = freq*sampSize
                int topLim = Math.ceil(numReps)
                if (topLim==0) return

                String myRep = ""
                for (int mut=0;mut<numMuts;mut++) {
                    if ((locus & (int)Math.pow(2, mut))>0) {
                        myRep += " 02"
                    }
                    else {
                        myRep += " 01"
                    }
                }
                for(int i=0; i<topLim; i++) {
                    oF << "pop${currPop}ind${currInd}, ${myRep}\n"
                    currInd ++
                }
            }
            currPop ++
        }
    }



    static void plotCase(PolicyResult result,
        List<List<Integer>> drugMuts, int epiType, String dir, String fName) {
        Map<String,List<Double>> swap = [:]
        File oF = new File(dir + File.separator + fName)
        oF.delete()
        result.genFreqs.eachWithIndex { gens, g ->
            gens.eachWithIndex {freq, locus ->
                oF << locus + " " + freq + "\n"
            }
        }
    }


    static void plotCasePheno(PolicyResult result,
        List<List<Integer>> drugMuts, int epiType, String dir, String fName) {
        Map<String,List<Double>> swap = [:]
        Map<Integer,List<Double>> fullSwap = [:]
        File oF = new File(dir + File.separator + fName)
        oF.delete()
        File aoF = new File(dir + File.separator + "all" + fName)
        aoF.delete()
        result.genFreqs.eachWithIndex { gens, g ->
            gens.eachWithIndex {freq, locus ->
                String key = ""
                drugMuts.each { muts ->
                    String drugKey
                    if (epiType == 0 | true) {
                        drugKey = "R"
                        boolean sensitive = true
                        muts.each { mut ->
                            if ((locus & (int)Math.pow(2, mut))>0) {
                                sensitive = false
                            }
                            else {
                                drugKey = "T"
                            }
                        }
                        if (sensitive) drugKey = "S"
                    }
                    else {
                        drugKey = "S"
                        muts.each { mut ->
                            if ((locus & (int)Math.pow(2, mut))>0) {
                                drugKey = "R"
                            }
                        }
                    }
                    key += drugKey
                }
                if (swap[key] == null) swap[key] = []
                swap[key][g] = (swap[key][g] ?: 0D) + freq
                if (fullSwap[locus] == null) fullSwap[locus] = []
                fullSwap[locus][g] = freq
            }
        }

        swap.each { tf ->
            oF << tf.key
            for (i in 0..(tf.value.size()-1)) {
                oF << " " + tf.value[i]
            }
            oF << "\n"
        }
        fullSwap.each { tf ->
            aoF << tf.key
            for (i in 0..(tf.value.size()-1)) {
                aoF << " " + tf.value[i]
            }
            aoF << "\n"
        }

    }



    static void plotWinner(
        int GENS, double THRESHOLD,
        int epiType, int[][] drugMuts,
        Simulation sim,
        String origPol, String origPolName,
        String comparePol, String comparePolName,
        boolean doTS, int minSep, double percentSep, String fName) {
        //Assuming that for each MFT there is an equivalent Thres
        Simulation mftSim = sim.getSimPostfix(origPol)
        Map<ResultClassifier,PolicyResult> mftResults = mftSim.results
        List<Double> dfMFT =  []
        List<Double> sMFT  =  []
        List<Double> dfThr =  []
        List<Double> sThr  =  []
        List<Double> dfTie =  []
        List<Double> sTie  =  []
	    File oF = new File(fName)
	    oF.delete()
        Table table = new Table(System.out)
        mftResults.each {
            ResultClassifier rc = it.key
            Simulation dsSim
            if (doTS) {
                dsSim = sim.getSimTS(rc.ts, rc.ts).getSimDrugFreq(rc.dFreq, rc.dFreq)
            }
            else {
                dsSim = sim.getSimS(rc.s, rc.s).getSimDrugFreq(rc.dFreq, rc.dFreq)
            }
            double funda
            if (doTS) {
                funda = rc.ts
            }
            else {
                funda = rc.s
            }
            List<Integer> ci = table.countWinner(GENS, dsSim, (List)drugMuts,
                THRESHOLD, epiType, [2,1,0],
                origPol, comparePol, minSep, percentSep)
            if (ci[0]>0) {
                dfMFT += rc.dFreq
                sMFT  += funda
            }
            else if (ci[1]>0) {
                dfThr += rc.dFreq
                sThr  += funda
            }
            else {
                dfTie += rc.dFreq
                sTie  += funda
            }

        }
        oF << origPolName + "\n"
        dumpList(oF, dfMFT)
        dumpList(oF, sMFT)
        oF << comparePolName + "\n"
        dumpList(oF, dfThr)
        dumpList(oF, sThr)
        oF << "Tie\n"
        dumpList(oF, dfTie)
        dumpList(oF, sTie)
    }

}

