package malaria;

/**
 *
 * @author tra
 */

import java.io.*;
import java.util.*;

public class Persistence {
    String dumpDir;
    
    public Persistence(String dumpDir) {
        this.dumpDir = dumpDir;
    }
    
    public String getThrRotPath(String initialDir,
            GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, double replThres,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int minInterval,
            int epiType,
            int gens) {
        String drugsDir = initialDir + File.separator + gc.drugMuts.length;
        new File(drugsDir).mkdir();
        String mutateDir = drugsDir + File.separator + gc.mutate + gc.probIntro + gc.precentReplaced;
        new File(mutateDir).mkdir();
        String sDir = mutateDir + File.separator + gc.s;
        new File(sDir).mkdir();
        String tsDir = sDir + File.separator + gc.ts;
        new File(tsDir).mkdir();
        String isSIDir = tsDir + File.separator + gc.isSI;
        new File(isSIDir).mkdir();
        String freqsDir = isSIDir + File.separator + doubleFreqsToFName(initialFreqs);
        new File(freqsDir).mkdir();
        String clonesDir = freqsDir + File.separator + gc.totalClones;
        new File(clonesDir).mkdir();
        String replThresDir = clonesDir + File.separator + replThres;
        new File(replThresDir).mkdir();
        String drugMutsDir = replThresDir + File.separator + Persistence.drugMutsToFName(gc.drugMuts);
        new File(drugMutsDir).mkdir();
        String clonalMultsDir = drugMutsDir + File.separator + Persistence.clonalMultstoFName(clonalMults);
        new File(clonalMultsDir).mkdir();
        String resFreqsDir = clonalMultsDir + File.separator + doubleFreqsToFName(residualFreqs);
        new File(resFreqsDir).mkdir();
        String minIntervalDir = resFreqsDir + File.separator + minInterval;
        new File(minIntervalDir).mkdir();
        String epiDir = minIntervalDir + File.separator + epiType;
        new File(epiDir).mkdir();
        String fName = gens + "+" + drugFreq ;
        //System.out.println(replThresDir + File.separator + fName);
        return epiDir + File.separator + fName;
    }
    

    public String getThrRotFixPath(String initialDir,
            GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, int swapDuration,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String drugsDir = initialDir + File.separator + gc.drugMuts.length;
        new File(drugsDir).mkdir();
        String mutateDir = drugsDir + File.separator + gc.mutate + gc.probIntro + gc.precentReplaced;
        new File(mutateDir).mkdir();
        String sDir = mutateDir + File.separator + gc.s;
        new File(sDir).mkdir();
        String tsDir = sDir + File.separator + gc.ts;
        new File(tsDir).mkdir();
        String isSIDir = tsDir + File.separator + gc.isSI;
        new File(isSIDir).mkdir();
        String freqsDir = isSIDir + File.separator + doubleFreqsToFName(initialFreqs);
        new File(freqsDir).mkdir();
        String clonesDir = freqsDir + File.separator + gc.totalClones;
        new File(clonesDir).mkdir();
        String replThresDir = clonesDir + File.separator + swapDuration;
        new File(replThresDir).mkdir();
        String drugMutsDir = replThresDir + File.separator + Persistence.drugMutsToFName(gc.drugMuts);
        new File(drugMutsDir).mkdir();
        String clonalMultsDir = drugMutsDir + File.separator + Persistence.clonalMultstoFName(clonalMults);
        new File(clonalMultsDir).mkdir();
        String resFreqsDir = clonalMultsDir + File.separator + doubleFreqsToFName(residualFreqs);
        new File(resFreqsDir).mkdir();
        String epiDir = resFreqsDir + File.separator + epiType;
        new File(epiDir).mkdir();
        String fName = gens + "+" + drugFreq ;
        //System.out.println(replThresDir + File.separator + fName);
        return epiDir + File.separator + fName;
    }
    
    

    //USED FOR MFT AND CT
    String getMFTPath(
            String initialDir,
            GenomicConfiguration gc,
            double[] drugFreqs, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String drugsDir = initialDir + File.separator + gc.drugMuts.length;
        new File(drugsDir).mkdir();
        String mutateDir = drugsDir + File.separator + gc.mutate + gc.probIntro + gc.precentReplaced;
        new File(mutateDir).mkdir();
        String sDir = mutateDir + File.separator + gc.s;
        new File(sDir).mkdir();
        String tsDir = sDir + File.separator + gc.ts;
        new File(tsDir).mkdir();
        String isSIDir = tsDir + File.separator + gc.isSI;
        new File(isSIDir).mkdir();
        String freqsDir = isSIDir + File.separator + doubleFreqsToFName(initialFreqs);
        new File(freqsDir).mkdir();
        String clonesDir = freqsDir + File.separator + gc.totalClones;
        new File(clonesDir).mkdir();
        String drugMutsDir = clonesDir + File.separator + Persistence.drugMutsToFName(gc.drugMuts);
        new File(drugMutsDir).mkdir();
        String clonalMultsDir = drugMutsDir + File.separator + Persistence.clonalMultstoFName(clonalMults);
        new File(clonalMultsDir).mkdir();
        String resFreqsDir = clonalMultsDir + File.separator + doubleFreqsToFName(residualFreqs);
        new File(resFreqsDir).mkdir();
        String epiDir = resFreqsDir + File.separator + epiType;
        new File(epiDir).mkdir();
        String fName = gens + "+" + Persistence.doubleFreqsToFName(drugFreqs) ;                        
        //System.out.println(resFreqsDir + File.separator + fName);
        return epiDir + File.separator + fName;
    }

    void dumpResults(PolicyResult results, String fName) {
        try {
            ObjectOutputStream oos = new ObjectOutputStream(
                new java.io.FileOutputStream(fName, false));
            oos.writeObject(results);
            oos.close();
        }
        catch (IOException ioe) {
            ioe.printStackTrace();
        }
        
    }

    PolicyResult loadResults(String fName) {
        PolicyResult results = null;
        try {
            ObjectInputStream ois = new ObjectInputStream(
                new FileInputStream(fName));
            results = (PolicyResult)ois.readObject();
            ois.close();
        }
        catch (FileNotFoundException fnfe) {
        }
	catch (EOFException eofe) {
	    //Probably was saved by another JRE version
	}
        catch (IOException ioe) {
            ioe.printStackTrace();
        }
        catch (ClassNotFoundException cnfe) {
            cnfe.printStackTrace();
        }
        return results;
    }
    
    public void runMFT(GenomicConfiguration gc,
            double[] drugFreqs, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "MFT";
        new File(dummyDir).mkdir();
        String fName = this.getMFTPath(dummyDir, gc, drugFreqs, initialFreqs,
                clonalMults, residualFreqs, epiType, gens);
        //System.out.println("Persistance: dName " + fName);
        PolicyResult results = Policy.runMFT(gc,
            drugFreqs, initialFreqs,
            clonalMults, residualFreqs, epiType, gens);
        dumpResults(results, fName);
    }

    
    public void runThresholdRotate(GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, double replThres,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int  minInterval,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "ThresRot";
        new File(dummyDir).mkdir();
        String fName = getThrRotPath(dummyDir, gc, drugFreq,
                numDrugs, initialFreqs, replThres,
                clonalMults, residualFreqs, minInterval, epiType, gens);
        //System.out.println("Persistance: rrame " + fName);
        PolicyResult results = Policy.runThresholdRotate(gc,
            drugFreq, numDrugs, initialFreqs, replThres,
            clonalMults, residualFreqs, minInterval, epiType, gens);
        dumpResults(results, fName);
    }

    public void runCT(GenomicConfiguration gc,
            double drugFreq, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double residualFreq,
            int epiType,
            int gens) {
        double[] drugFreqs = new double[1];
        drugFreqs[0] = drugFreq;
        double[] residualFreqs = new double[1];
        residualFreqs[0] = residualFreq;
        String dummyDir = dumpDir + File.separator + "CT";
        new File(dummyDir).mkdir();
        String fName = this.getMFTPath(dummyDir, gc, drugFreqs, initialFreqs,
                clonalMults, residualFreqs, epiType, gens);
        //System.out.println("Persistance: dName " + fName);
        PolicyResult results = Policy.runCT(gc,
            drugFreq, initialFreqs,
            clonalMults, residualFreq, epiType, gens);
        dumpResults(results, fName);
    }




    double[][] correctGens(double[][] genFreqs) {
        int cutGen = 0;
        for(int i=0; i<genFreqs.length; i++) {
            double sum = 0.0;
            for (int cl=0; cl<genFreqs[i].length; cl++) {
                sum += genFreqs[i][cl];
            }
            if (sum<0.9) {
                cutGen = i;
                break;
            }
        }
        double[][] newGenFreqs = new double[cutGen][genFreqs[0].length];
        for (int i=0;i<cutGen;i++) {
            for (int cl=0; cl<genFreqs[i].length; cl++) {
                newGenFreqs[i][cl] = genFreqs[i][cl];
            }            
        }
        return newGenFreqs;
    }
    
   
    
    public void runThresholdRotateFixed(GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, int swapDuration,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "ThresRotFix";
        new File(dummyDir).mkdir();
        String fName = getThrRotFixPath(dummyDir, gc, drugFreq,
                numDrugs, initialFreqs, swapDuration, clonalMults,
                residualFreqs, epiType, gens);
        //System.out.println("Persistance: rrame " + fName);
        PolicyResult results = Policy.runThresholdRotateFixed(gc,
            drugFreq, numDrugs, initialFreqs, swapDuration,
            clonalMults, residualFreqs, epiType, gens);
        dumpResults(results, fName);
    }


    public PolicyResult retrieveThresholdRotate(GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, double replThres,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int  minInterval,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "ThresRot";
        new File(dummyDir).mkdir();
        String fName = getThrRotPath(dummyDir, gc, drugFreq,
                numDrugs, initialFreqs, replThres,
                clonalMults, residualFreqs, minInterval, epiType, gens);
        File aFile = new File(fName);
        //System.out.println("Persistance: rName " + fName);
        PolicyResult results = loadResults(fName);
        //if (!aFile.exists()) {
        if (results==null || gc.mutate) {
            runThresholdRotate(gc, drugFreq, numDrugs,
                    initialFreqs, replThres,
                    clonalMults, residualFreqs, minInterval, epiType, gens);
        }
	results = loadResults(fName);
        return results;
    }

    public PolicyResult retrieveThresholdRotateFixed(GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, int swapDrug,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "ThresRotFix";
        new File(dummyDir).mkdir();
        String fName = getThrRotFixPath(dummyDir, gc, drugFreq,
                numDrugs, initialFreqs, swapDrug,
                clonalMults, residualFreqs, epiType, gens);
        File aFile = new File(fName);
        //System.out.println("Persistance: rName " + fName);
        PolicyResult results = loadResults(fName);
        //if (!aFile.exists()) {
        if (results==null || gc.mutate) {
            runThresholdRotateFixed(gc, drugFreq, numDrugs,
                    initialFreqs, swapDrug,
                    clonalMults, residualFreqs, epiType, gens);
        }
	results = loadResults(fName);
        return results;
    }

    
    public PolicyResult retrieveMFT(GenomicConfiguration gc,
            double[] drugFreqs, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int epiType,
            int gens) {
        String dummyDir = dumpDir + File.separator + "MFT";
        new File(dummyDir).mkdir();
        String fName = getMFTPath(dummyDir, gc, drugFreqs, initialFreqs,
                clonalMults, residualFreqs, epiType, gens);
        //System.out.println("Persistance: fName " + fName);
        PolicyResult results;
        results = loadResults(fName);
        //if (!aFile.exists()) {
        if (results==null || gc.mutate) {
            runMFT(gc, drugFreqs, initialFreqs, clonalMults, residualFreqs,
                    epiType, gens);
        }
        results = loadResults(fName);
        return results;
    }


    public PolicyResult retrieveCT(GenomicConfiguration gc,
            double drugFreq, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double residualFreq,
            int epiType,
            int gens) {
        double[] drugFreqs = new double[1];
        drugFreqs[0] = drugFreq;
        double[] residualFreqs = new double[1];
        residualFreqs[0] = residualFreq;
        String dummyDir = dumpDir + File.separator + "CT";
        new File(dummyDir).mkdir();
        String fName = getMFTPath(dummyDir, gc, drugFreqs, initialFreqs,
                clonalMults, residualFreqs, epiType, gens);
        //System.out.println("Persistance: fName " + fName);
        File aFile = new File(fName);
        PolicyResult results = loadResults(fName);
        //if (!aFile.exists()) {
        if (results==null || gc.mutate) {
            runCT(gc, drugFreq, initialFreqs, clonalMults, residualFreq,
                    epiType, gens);
        }
        results = loadResults(fName);
        return results;
    }


            
    public static String doubleFreqsToFName(double[] drugFreqs) {
        String part="";
        for (double freq : drugFreqs) {
            part += freq + ":";
        }
        part = part.substring(0, part.length()-1);
        return "" + part.hashCode();
    }

    public static String drugMutsToFName(int[][] drugMuts) {
        String part="";
        for (int[] dM : drugMuts) {
            for (int mut : dM) {
                part += mut;
                part += ":";
            }
            part = part.substring(0, part.length()-1);
            part += "=";
        }
        part = part.substring(0, part.length()-1);
        return "" + part.hashCode();
    }
    
    public static String clonalMultstoFName(HashMap<Integer, Double> cmMap) {
        String part="";
        List<Integer> CMs=new ArrayList<Integer>(cmMap.keySet());
        java.util.Collections.sort(CMs);
        for (int cm : CMs) {
            part += cm + ":" + cmMap.get(cm);
            part += "=";
        }
        part = part.substring(0, part.length()-1);
        return "" + part.hashCode();
    }

}
