package malaria;

import java.util.*;

public class Policy {
    public final static double MIN_FREQ_STOP = -1;//Never stopping
    public final static double MAX_FREQ_STOP = +2; //Never stopping
    //public final static double MIN_FREQ_STOP = 0.00001;
    //public final static double MAX_FREQ_STOP = 0.99999;
    public final static int MIN_GENS = Integer.MAX_VALUE;
    //public final static int MIN_GENS = 30;
    
    public static PolicyResult runMFT(GenomicConfiguration gc,
            double[] drugFreqs, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs, int epiType,
            int gens) {
        //Strictly speaking, at first generation there are no residual freqs
        //  but it is just the initial case and this is better for
        //  Threshold rotate.
        double[] freqs = initialFreqs;
        double[][] results = new double[gens][gc.totalClones];
        results[0] = freqs;
        Recombination rc = new Recombination(gc, epiType);
        int genOK = 0;
        for(int gen=1; gen<gens; gen++) {
            freqs = rc.apply(freqs, drugFreqs, clonalMults, residualFreqs);
            results[gen] = freqs;
            /*
             * Computation will be halted if:
             *    generation> 30 AND
             *      sensitive frequency is below 0.01 or above 0.99 for 10 gens
             */
            if (freqs[0]>MIN_FREQ_STOP && freqs[0]<MAX_FREQ_STOP) {
                genOK = gen;
            }
            else {
                if (gen>genOK+10 && gen > MIN_GENS) {
                    double[][] newResults = new double[gen+1][gc.totalClones];
                    for(int gi=0; gi<=gen; gi++) {
                        for(int ci=0;ci< gc.totalClones; ci++) {
                            newResults[gi][ci] = results[gi][ci];
                        }
                    }
                    results = newResults;
                    //System.out.println("Policy: stopping at " + gen);
                    break;
                }
            }
        }
        return new PolicyResult(results);
    }

    //There is a problem when drug is rotated
    //  Frequency of resistant is severely depressed, a boost might be needed
    //     Think immigration of resistants
    public static PolicyResult runThresholdRotate(GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, double replThres,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs,
            int minInterval, int epiType,
            int gens) {
        double[] freqs = initialFreqs;
        Vector<Integer> swapTimesV = new Vector<Integer>();
        int[] swapTimes;
        Vector<Integer> usedDrugsV = new Vector<Integer>();
        int[] usedDrugs;
        int mftGen = -1;
        double[][] results = new double[gens][gc.totalClones];
        results[0] = freqs;
        double[] drugFreqs = new double[numDrugs];
        double[] myResidualFreqs = new double[residualFreqs.length];
        int currentDrug = 0;
        swapTimesV.add(0);
        usedDrugsV.add(0);
        drugFreqs[currentDrug] = drugFreq;
        Recombination rc = new Recombination(gc, epiType);
        int genOK=0;
        int gen;
        //myResiduals are 0 at start
forGen:
        for(gen=1; gen<gens; gen++) {
            int[] dM;
            double resistFreq = 0;
            freqs = rc.apply(freqs, drugFreqs, clonalMults, myResidualFreqs);
            dM = gc.drugMuts[currentDrug];
            for(int clone=0;clone<freqs.length; clone++) {
		if (epiType == 0){
                    boolean isResistant = true;
                    for (int mut : dM) {
                        if (((int)Math.pow(2, mut) & clone) == 0) {
                            isResistant = false;
                            break;
                        }
                    }
                    if (isResistant) resistFreq += freqs[clone];
		}
		else if (epiType == 1){
                    boolean isResistant = false;
                    for (int mut : dM) {
                        if (((int)Math.pow(2, mut) & clone) != 0) {
                            isResistant = true;
                            break;
                        }
                    }
                    if (isResistant) resistFreq += freqs[clone];
		}
		else if (epiType == 2){
                    boolean isFull = true;
		    boolean isPart = false;
                    for (int mut : dM) {
                        if (((int)Math.pow(2, mut) & clone) != 0) {
                            isPart = true;
                        }
			else {
			    isFull = false;
			}
                    }
		    if (isFull) {
                        resistFreq += freqs[clone];
		    }
		    else if (isPart) {
                        resistFreq += freqs[clone]/2;
		    }
		}
		else if (epiType == 3){
                    boolean isFirst = ((int)Math.pow(2, dM[0]) & clone) != 0;
		    boolean isFull = true;
                    for (int mut : dM) {
                        if (((int)Math.pow(2, mut) & clone) == 0) {
			    isFull = false;
			}
                    }
		    if (isFull) {
                        resistFreq += freqs[clone];
		    }
		    else if (isFirst) {
                        resistFreq += freqs[clone]/2;
		    }
		}
            }
            results[gen] = freqs;
            for (int i= 0; i<residualFreqs.length; i++) {
                if (currentDrug ==i) {
                    myResidualFreqs[i] = residualFreqs[i];
                }
                else {
                    myResidualFreqs[i] = 0;
                }
            }
                
            if(resistFreq>replThres) {
                if (gen-swapTimesV.get(swapTimesV.size()-1)<minInterval) {
                    //Policy is no more... Trying MFT for the last part
                    PolicyResult mftFinal;
                    mftGen = gen;
                    double[] mftDrugFreqs = new double[gc.numDrugs];
                    for (int i=0; i<numDrugs; i++) {
                        mftDrugFreqs[i] = drugFreq/gc.numDrugs;
                    }
                    mftFinal = runMFT(gc, drugFreqs, freqs, clonalMults,
                            residualFreqs, epiType, gens-gen);
                    for (int mftI=1; mftI<mftFinal.genFreqs.length; mftI++) {
                        //mftI=0 is the current one
                        results[gen+mftI] = mftFinal.genFreqs[mftI];
                    }
                    //System.out.println("Pol: Done MFT on THR break " + gen + " " + (mftFinal.genFreqs.length-1));
                    gen += mftFinal.genFreqs.length-1;
                    break forGen;
                }
                drugFreqs[currentDrug] = 0;
                currentDrug ++;
                if (currentDrug>=numDrugs) currentDrug = 0;
                drugFreqs[currentDrug] = drugFreq;
                swapTimesV.add(gen);
                usedDrugsV.add(currentDrug);
                //System.out.println(gen + " Swapping current drug to " + (currentDrug+1));
                //System.out.println(Recombination.freqsToString(freqs));
            }
            if (freqs[0]>MIN_FREQ_STOP && freqs[0]<MAX_FREQ_STOP) {
                genOK = gen;
            }
            else {
                if (gen>genOK+10 && gen > MIN_GENS) {
                    //System.out.println("Policy: stopping at " + gen);
                    break forGen;
                }
            }
        }        
        swapTimes = new int[swapTimesV.size()];
        usedDrugs = new int[usedDrugsV.size()];
        for(int i=0; i<swapTimes.length; i++) {
            swapTimes[i] = swapTimesV.get(i);
            usedDrugs[i] = usedDrugsV.get(i);
        }
        //System.out.println("Policy: recording " + gen);
        double[][] newResults = new double[gen][gc.totalClones];
        for (int i=0; i<gen; i++) {
            newResults[i] = results[i];
        }
        return new PolicyResult(newResults, swapTimes, usedDrugs, mftGen);
    }

    
    //There is a problem when drug is rotated
    //  Frequency of resistant is severely depressed, a boost might be needed
    //     Think immigration of resistants
    public static PolicyResult runThresholdRotateFixed(
            GenomicConfiguration gc,
            double drugFreq, int numDrugs,
            double[] initialFreqs, int swapDuration,
            HashMap<Integer, Double> clonalMults,
            double[] residualFreqs, int epiType,
            int gens) {
        double[] freqs = initialFreqs;
        Vector<Integer> swapTimesV = new Vector<Integer>();
        int[] swapTimes;
        Vector<Integer> usedDrugsV = new Vector<Integer>();
        int[] usedDrugs;
        double[][] results = new double[gens][gc.totalClones];
        results[0] = freqs;
        double[] drugFreqs = new double[numDrugs];
        int currentDrug = 0;
        swapTimesV.add(0);
        usedDrugsV.add(0);
        drugFreqs[currentDrug] = drugFreq;
        Recombination rc = new Recombination(gc, epiType);
        int genOK=0;
        for(int gen=1; gen<gens; gen++) {
            freqs = rc.apply(freqs, drugFreqs, clonalMults, residualFreqs);
            results[gen] = freqs;
            if(gen > 0 && gen % swapDuration == 0) {
                drugFreqs[currentDrug] = 0;
                currentDrug ++;
                if (currentDrug>=numDrugs) currentDrug = 0;
                drugFreqs[currentDrug] = drugFreq;
                swapTimesV.add(gen);
                usedDrugsV.add(currentDrug);
                //System.out.println(gen + " Swapping current drug to " + (currentDrug+1));
                //System.out.println(Recombination.freqsToString(freqs));
            }
            if (freqs[0]>MIN_FREQ_STOP && freqs[0]<MAX_FREQ_STOP) {
                genOK = gen;
            }
            else {
                if (gen>genOK+10 && gen > MIN_GENS) {
                    //System.out.println("Policy: stopping at " + gen);
                    break;
                }
            }
        }        
        swapTimes = new int[swapTimesV.size()];
        usedDrugs = new int[usedDrugsV.size()];
        for(int i=0; i<swapTimes.length; i++) {
            swapTimes[i] = swapTimesV.get(i);
            usedDrugs[i] = usedDrugsV.get(i);
        }
        return new PolicyResult(results, swapTimes, usedDrugs, -1);
    }

    public static PolicyResult runCT(GenomicConfiguration gc,
            double drugFreq, double[] initialFreqs,
            HashMap<Integer, Double> clonalMults,
            double residualFreq, int epiType,
            int gens) {
        //Strictly speaking, at first generation there are no residual freqs
        //  but it is just the initial case and this is better for
        //  Threshold rotate.
        double[] freqs = initialFreqs;
        double[][] results = new double[gens][gc.totalClones];
        results[0] = freqs;
        GenomicConfiguration ctGc;
        int[][] ctDrugMuts = new int[1][gc.drugMuts.length*gc.drugMuts[0].length];
        for (int drug=0; drug<gc.drugMuts.length; drug++) {
            for (int locus=0; locus<gc.drugMuts[drug].length; locus++) {
                ctDrugMuts[0][drug*gc.drugMuts[0].length + locus] = gc.drugMuts[drug][locus];
            }
        }
        ctGc = new GenomicConfiguration(gc.s, gc.ts, gc.isSI, ctDrugMuts,
                gc.mutate, gc.probIntro, gc.precentReplaced);
        Recombination rc = new Recombination(ctGc, epiType);
        int genOK = 0;
        double[] drugFreqs = new double[1];
        drugFreqs[0] = drugFreq;
        double[] residualFreqs = new double[1];
        residualFreqs[0] = residualFreq;
        for(int gen=1; gen<gens; gen++) {
            freqs = rc.apply(freqs, drugFreqs, clonalMults, residualFreqs);
            results[gen] = freqs;
            /*
             * Computation will be halted if:
             *    generation> 30 AND
             *      sensitive frequency is below 0.01 or above 0.99 for 10 gens
             */
            if (freqs[0]>MIN_FREQ_STOP && freqs[0]<MAX_FREQ_STOP) {
                genOK = gen;
            }
            else {
                if (gen>genOK+10 && gen > MIN_GENS) {
                    double[][] newResults = new double[gen+1][gc.totalClones];
                    for(int gi=0; gi<=gen; gi++) {
                        for(int ci=0;ci< gc.totalClones; ci++) {
                            newResults[gi][ci] = results[gi][ci];
                        }
                    }
                    results = newResults;
                    //System.out.println("Policy: stopping at " + gen);
                    break;
                }
            }
        }
        return new PolicyResult(results);
    }
    
}
