package malaria

/**
 *
 * @author tra
 */
class Simulation {
    //static final int MIN_GENS = 10
    int gens
    Map<ResultClassifier, PolicyResult> results
    Persistence writer
    int ldType
    double initialFreq
    double threshold
    boolean onlyRun
    int epiType
    boolean mutate
    double probIntro
    double percentReplaced

    public Simulation(int gens,
        Persistence writer,
        int ldType, double initialFreq, double threshold,
        boolean onlyRun, int epiType,
        boolean mutate, double probIntro, double percentReplaced) {
        this.gens = gens
        this.writer = writer
        this.ldType = ldType
        this.initialFreq = initialFreq
        this.threshold = threshold
        this.onlyRun = onlyRun
        this.epiType = epiType
        this.mutate = mutate
        this.probIntro = probIntro
        this.percentReplaced = percentReplaced
        results = new HashMap<ResultClassifier, PolicyResult>()
    }

    double[] getFreqs(GenomicConfiguration gc) {
        if (ldType<0) {
            return gc.getFreqsFixedForDrug(initialFreq)// 0.003
        }
        else if (ldType==0) {
            return gc.getFreqsLinkEqDrug(1-(gc.numDrugs* initialFreq))
        }
        else {
            return gc.getFreqsMultiForDrug(1-(gc.numDrugs* initialFreq))
        }
    }

    //rcDrugFreq is just for result classfier purposes, it should
    //  contain the expected used value (which might differ for real)
    PolicyResult doMFT(int numDrugs, double s, double ts, boolean isSI,
        double rcDFreq, double[] drugFreqs,
        HashMap<Integer, Double> clonalMults, int locDrug,
        double[] residualFreqs, boolean isACT) {
        GenomicConfiguration gc
        if (isACT) {
            gc = GenomicConfiguration.getACT(s, ts, isSI, numDrugs,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        else {
            gc = GenomicConfiguration.getStandard(s, ts, isSI, numDrugs, locDrug,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        //double[] drugFreqs = [dFreq/numDrugs] * numDrugs
        double[] freqs = getFreqs(gc)
        PolicyResult pr = writer.retrieveMFT(gc, drugFreqs, freqs,
            clonalMults, residualFreqs, epiType, gens)
        ResultClassifier rc = new ResultClassifier(
            clonalMults, numDrugs, rcDFreq, s, ts, isSI,
            locDrug, "MFT", residualFreqs, -1)
        if (!onlyRun) results[rc] = pr
        return pr
    }


    //rcDrugFreq is just for result classfier purposes, it should
    //  contain the expected used value (which might differ for real)
    PolicyResult doCT(int numDrugs, double s, double ts, boolean isSI,
        double drugFreq, HashMap<Integer, Double> clonalMults, int locDrug,
        double residualFreq, boolean isACT) {
        GenomicConfiguration gc
        if (isACT) {
            gc = GenomicConfiguration.getACT(s, ts, isSI, numDrugs,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        else {
            gc=GenomicConfiguration.getStandard(s, ts, isSI, numDrugs, locDrug,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        //double[] drugFreqs = [dFreq/numDrugs] * numDrugs
        double[] freqs = getFreqs(gc)
        PolicyResult pr = writer.retrieveCT(gc, drugFreq, freqs,
            clonalMults, residualFreq, epiType, gens)
        ResultClassifier rc = new ResultClassifier(
            clonalMults, numDrugs, drugFreq, s, ts, isSI,
            locDrug, "CT", (double[])[residualFreq], -1)
        if (!onlyRun) results[rc] = pr
        return pr
    }



    PolicyResult doThresRot(int numDrugs, double s, double ts,
        boolean isSI, double dFreq,
        HashMap<Integer, Double> clonalMults, int locDrug,
        double[] residualFreqs, int minInterval, boolean isACT) {
        GenomicConfiguration gc
        if (isACT) {
            gc = GenomicConfiguration.getACT(s, ts, isSI, numDrugs,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        else {
            gc=GenomicConfiguration.getStandard(s, ts, isSI, numDrugs, locDrug,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        double[] freqs = getFreqs(gc)
        PolicyResult pr = writer.retrieveThresholdRotate(gc, dFreq, numDrugs,
            freqs, threshold, clonalMults, residualFreqs, minInterval,
            epiType, gens)
        ResultClassifier rc = new ResultClassifier(clonalMults, numDrugs,
            dFreq, s, ts, isSI, locDrug, "ThresRot", residualFreqs, minInterval)
        if (!onlyRun) results[rc] = pr
        return pr
    }

    PolicyResult doThresRotFixed(int numDrugs, double s, double ts,
        boolean isSI, double dFreq, HashMap<Integer, Double> clonalMults,
        int locDrug, double[] residualFreqs, boolean isACT, int rotation) {
        GenomicConfiguration gc
        if (isACT) {
            gc = GenomicConfiguration.getACT(s, ts, isSI, numDrugs,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        else {
            gc=GenomicConfiguration.getStandard(s, ts, isSI, numDrugs, locDrug,
                this.mutate, this.probIntro, this.percentReplaced)
        }
        double[] freqs = getFreqs(gc)
        PolicyResult pr = writer.retrieveThresholdRotateFixed(gc, dFreq, numDrugs,
            freqs, rotation, clonalMults, residualFreqs, epiType, gens)
        ResultClassifier rc = new ResultClassifier(clonalMults, numDrugs,
            dFreq, s, ts, isSI, locDrug, "ThresRotFixed", residualFreqs, -1)
        if (!onlyRun) results[rc] = pr
        return pr
    }

    
    Simulation getSimCM(Map<Integer, Double> cm) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.cm==cm) {
                simResults.results[it.key] = it.value
            }
        }
        //println "Simulation: " + results.size() + " " + simResults.results.size()
        return simResults
    }

    Simulation getSimS(double minS, double maxS) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.s>=minS && it.key.s<=maxS) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }

    Simulation getSimTS(double minTS, double maxTS) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.ts>=minTS && it.key.ts<=maxTS) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }

    Simulation getSimNumDrugs(int numDrugs) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.numDrugs == numDrugs) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }

    Simulation getSimLocDrug(int locDrug) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.locDrug == locDrug) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }
    
    Simulation getSimDrugFreq(double minD, double maxD) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.dFreq>=minD && it.key.dFreq<=maxD) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }

    Simulation getSimPrefix(String prefix) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.policy.startsWith(prefix)) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }

    Simulation getSimPostfix(String postfix) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.policy.endsWith(postfix)) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }
    
    Simulation getSimS(List<Double> Ss) {
        Simulation simResults = new Simulation(this.gens, this.writer,
            this.ldType, this.initialFreq, this.threshold, false, epiType,
            this.mutate, this.probIntro, this.percentReplaced)
        results.each {
            if (it.key.s in Ss) {
                simResults.results[it.key] = it.value
            }
        }
        return simResults
    }
 
    Simulation getSimTS(List<Double> TSs) {
         Simulation simResults = new Simulation(this.gens, this.writer,
             this.ldType, this.initialFreq, this.threshold, false, epiType,
             this.mutate, this.probIntro, this.percentReplaced)
         results.each {
             if (it.key.ts in TSs) {
                 simResults.results[it.key] = it.value
             }
         }
         return simResults
     }

     Simulation getSimDrugFreq(List<Double> dFreqs) {
         Simulation simResults = new Simulation(this.gens, this.writer,
             this.ldType, this.initialFreq, this.threshold, false, epiType,
             this.mutate, this.probIntro, this.percentReplaced)
         results.each {
             if (it.key.dFreq in dFreqs) {
                 simResults.results[it.key] = it.value
             }
         }
         return simResults
     }
 
     PolicyResult getClassifier(ResultClassifier rc) {
         results.each {
             if (it.key.equals(rc)) return it.value
         }
         return null
     }
}
