package malaria

/**
 *
 * @author tra
 */
class SimulationRunner {
    Simulation sim
    List<HashMap<Integer,Double>> workMults
    List<Double> resLevels
    List<Double> TSs
    List<Double> Ss
    List<Double> dFreqs
    boolean isSI
    StatusPane pane
    ChartPane chartPane

    public SimulationRunner(Simulation sim,
        List<HashMap<Integer,Double>> workMults, List<Double> resLevels,
        List<Double> TSs, List<Double> Ss, List<Double> dFreqs,
        boolean isSI
    ) {
        this.sim = sim
        this.workMults = workMults
        this.resLevels = resLevels
        this.TSs = TSs
        this.Ss = Ss
        this.dFreqs = dFreqs
        this.isSI = isSI
        this.pane = null
    }

    List scramble(List list) {
        List ret = []
        List<Integer> usedPos = []
        Random r = new Random()
        while (ret.size()< list.size()) {
            int pos = r.nextInt(list.size())
            while (pos in usedPos) {
                pos = r.nextInt(list.size())
            }
            usedPos += pos
            ret += list[pos]
        }
        return ret
    }

    void setPane(StatusPane pane) {
        this.pane = pane
    }

    void setChartPane(ChartPane pane) {
        this.chartPane = pane
    }


    void runACT(int numDrugs, int minInterval, int policy, int rotation) {
        runAll(numDrugs, -1, minInterval, true, policy, rotation)
    }

    void runStandard(int numDrugs, int locPerDrug, int minInterval,
        int policy, int rotation) {
        runAll(numDrugs, locPerDrug, minInterval, false, policy, rotation)
    }

    void runAll(int numDrugs, int locPerDrug, int minInterval, boolean isACT,
        int policy, int rotation) {
        for (clonalMults in scramble(workMults)) {
            for (resLevel in scramble(resLevels)) {
                for (ts in scramble(TSs)) {
                    for (s in scramble(Ss)) {
                        //println "s/ts " + s + "/" + ts
                        for (dFreq in scramble(dFreqs)) {
                            double[] d1 = [resLevel]
                            double[] d2 = [resLevel/2, resLevel/2]
                            double[] d3 = [resLevel/3, resLevel/3, resLevel/3]
                            double[] d4 = [resLevel/4, resLevel/4, resLevel/4, resLevel/4]
                            double[] d
                            //FIXME awful code....
                            if (numDrugs == 1) {
                                d = d1
                            }
                            if (numDrugs == 2) {
                                d = d2
                            }
                            if (numDrugs == 3) {
                                d = d3
                            }
                            if (numDrugs == 4) {
                                d = d4
                            }
                            PolicyResult pr
                            if (policy==0) {
                                if (rotation>0) {
                                    pr = sim.doThresRotFixed(numDrugs, s, ts, isSI,
                                        dFreq, clonalMults, locPerDrug, d,
                                        isACT, rotation)

                                }
                                else {
                                    pr = sim.doThresRot(numDrugs, s, ts, isSI,
                                        dFreq, clonalMults, locPerDrug, d,
                                        minInterval, isACT)
                                }
                            }
                            else if (policy==1) {
                                pr = sim.doMFT(numDrugs, s, ts, isSI,
                                    dFreq, (double[])([dFreq/numDrugs]*numDrugs),
                                    clonalMults, locPerDrug, d, isACT)
                            }
                            else if (policy==2) {
                                pr = sim.doCT(numDrugs, s, ts, isSI, dFreq,
                                    clonalMults, locPerDrug, resLevel, isACT)
                            }
                            if (pane!=null) pane.increment()
                            if (chartPane!=null) chartPane.inform(s, dFreq, pr)
                        }
                    }
                }
            }
        }
    }
	
}

