package malaria

import javax.swing.JTextPane

/**
 *
 * @author tiago
 */
class StatusPane {
    JTextPane pane
    UI ui
    String stopped = "<font color=\"#EAC117\">Stopped</font>"
    String status = stopped
    int done = 0

    def template = { cost, loci, sims, envs ->
        String directory = System.getProperty("user.home") + File.separator + "ogarak"
        return """
<table align="center">
  <tr><td colspan="2" align="center"><b>ogaraK</b></td></tr>

  <tr><td width="110">Cost (relative, per environment, for max MOI)</td><td width="110">${cost}</td></tr>
  <tr><td>Loci</td><td>${loci}</td></tr>
  <tr><td>Number of simulations</td><td>${sims}</td></tr>
  <tr><td>Number of environments<br>(simultaneous in time)</td><td>${envs}</td></tr>
</table>
<table align="center" border="1">
<tr><td align="center">Simulations done</td>
<td align="center">${done}</td></tr>
<tr><td align="center">Status</td>
<td align="center">${status}</td></tr>
</table>

        """
    }

    public StatusPane(UI ui) {
        pane = new JTextPane()
        pane.setEditable(false)
        pane.setContentType("text/html")
        pane.setText(template(0, 0, 0, 0))
        this.ui = ui
    }

    public void start() {
        status = "<font color=\"red\">Running</font>"
        done = 0
        recompute()
    }

    public void stop() {
        status = "<font color=\"green\">Done/Stopped</font>"
        recompute()
    }


    public void generate() {
        status = "<font color=\"#F665AB\">Writing data</font>"
        done = 0
        recompute()
    }


    public void increment() {
        done++
        recompute()
    }


    public JTextPane getPane() {
        return pane
    }

    public void recompute() {
        int loci
        BigInteger cost
        int sims
        int envs

        int numDrugs = ui.numDrugs.selectedIndex + 1
        int locPerDrug = ui.locPerDrug.selectedIndex + 1
        if (ui.drugType.selectedIndex == 0) { //Standard
            loci = numDrugs * locPerDrug
        }
        else { //Art
            loci = numDrugs + 1
        }
        int maxMults = 0
        ui.workMults.each {
            if (it.key>maxMults & it.value>0) {
                maxMults = it.key
            }
        }
        cost = (BigInteger.ONE.plus(BigInteger.ONE)).pow(loci).pow(maxMults)
        //println cost
        int drugEnvs = ui.epistasis.selectedIndex <2 ? 1 : 2
        if (ui.policy.selectedIndex == 1) { //MFT
            envs = 1 + numDrugs * drugEnvs
        }
        else { //Rot (0) or Comb (2)
            envs = 1 + drugEnvs
        }

        sims = getRangeLen(ui.usage.text) * getRangeLen(ui.fitness.text)

        pane.setText(template(cost, loci, sims, envs))
        pane.paintImmediately(0,0,pane.getWidth(),pane.getHeight())
    }

    int getRangeLen(String range) {
        if (range.indexOf("/")==-1) {
            return 1
        }
        else {
            double startV
            double endV
            double stepV
            List<String> toks = range.split("/")
            startV = Double.parseDouble(toks[0])
            endV = Double.parseDouble(toks[1])
            stepV = Double.parseDouble(toks[2])
            List<Double> res = []
            double currentV = startV
            while (currentV<=endV){
                res += currentV
                currentV += stepV

            }
            return res.size()
        }
    }
	
}

