package malaria

/**
 *
 * @author tra
 */


class Table {
    PrintStream ps;

    public Table(PrintStream ps) {
        this.ps = ps
    }

    List sort (List results, List<Integer> fields) {
        List sorted = []
        while (results.size()>0) {
            int currMin=0
            results.eachWithIndex { e, i ->
                for (int f=0; f<fields.size(); f++) {
                    int fld = fields[f]
                    if (e[fld]>results[currMin][fld]) {
                        break
                    }
                    else if (e[fld]<results[currMin][fld]) {
                        currMin = i
                        break
                    }
                }
            }
            sorted += [results.remove(currMin)]
        }
        return sorted
        
    }


    List reportSensitiveThres(Simulation sims, List<Double> cuts,
        List<Integer> fields) {
        Map<ResultClassifier, PolicyResult> results = sims.results
        List unsortedResults = []
        List sortedResults
        results.each {
            double[][] freqs = it.value.genFreqs
            List<Integer> cutGens = []
            int currCut = 0
            int currGen = 0
            while (currCut < cuts.size()) {
                while (currGen<freqs.length && cuts[currCut]<freqs[currGen][0]) {
                    currGen++
                }
                cutGens += currGen
                currCut++
            }
            unsortedResults += [[it.key.dFreq, it.key.s, cutGens]]
        }

        sortedResults = sort(unsortedResults, fields)
        return sortedResults
    }

    //policy s, d
    List reportDuration(int gens,
        Simulation sims,
        List<List<Integer>> drugMuts, double thres,
        int epiType,
        List<Integer> fields) {
        Map<ResultClassifier, PolicyResult> results = sims.results

        int[][] drugArrMuts = new int[drugMuts.size()][drugMuts[0].size()]
        drugMuts.eachWithIndex {muts, id ->
            muts.eachWithIndex {mut, im ->
                drugArrMuts[id][im] = mut
            }
        }
        List unsortedResults = []
        List sortedResults = []
        results.each {
            int duration = 0;
            duration = Analysis.reportDurationStandard(gens,
            it.key, it.value, drugArrMuts, thres, epiType)
            unsortedResults += [[it.key.policy, it.key.dFreq, it.key.s,
                duration]]
        }
        sortedResults = sort(unsortedResults, fields)
        return sortedResults
    }

    void dumpDuration(int gens,
        Simulation sims,
        List<List<Integer>> drugMuts, double thres,
        int epiType,
        List<Integer> fields) {
        dump(reportDuration(gens, sims, drugMuts, thres, epiType, fields))
    }

    List reportWinner(int gens,
        Simulation sims,
        List<List<Integer>> drugMuts, double thres,
        int epiType,
        List<Integer> fields,
        String policy1, String policy2,
        int minSep, double percentSep) {
        List durList=reportDuration(gens, sims, drugMuts, thres, epiType, fields)
        HashMap<List<Double>, String> outcome = [:]
        boolean[] got  = new boolean[2]
        int[]     dur  = new int[2]
        double    s
        double    dFreq
 
        //println "Table: malaria " + durList
        for (elem in durList) {
            int pol
            if (elem[0].equals(policy1)) {
                pol = 0
            }
            else if (elem[0].equals(policy2)) {
                pol = 1
            }
            else {
                continue
            }
            got[pol]   = true
            dFreq      = elem[1]
            s          = elem[2]
            dur[pol]   = elem[3]
            if (got[0] && got[1]) {
                String thisOutcome
                got[0] = false
                got[1] = false
		/*
		 * We can either code that -1000 == -1 or not (i.e. -1 is better)
		 * currently equality is assumed
		 */
                if (dur[0] <= -1) { //put == for -1 is better
                    if (dur[1] <= -1) { //ibidem
                        thisOutcome = "Equal-1"
                    }
                    else {
                        thisOutcome =  policy1 + "-1"
                    }
                }
                else {
                    if (dur[1] == -1) {
                        thisOutcome =  policy2 + "-1"
                    }
                    else  if (dur[0] < -1) {
                        if (dur[1] < -1) {
                            thisOutcome = "Equal-gen"
                        }
                        else {
                            thisOutcome =  policy1 + "-gen"
                        }
                    }
                    else {
                        if (dur[1] < -1) {
                            thisOutcome =  policy2 + "-gen"
                        }
                        else {
                            if (dur[0]>dur[1]) {
                                if (dur[0]<=dur[1]+minSep) {
                                    thisOutcome = "FixedSep" + policy1
                                }
                                else if (dur[0]/dur[1]<1+percentSep) {
                                    thisOutcome = "PercentSep" + policy1
                                }
                                else {
                                    thisOutcome =  policy1
                                }
                            }
                            else if (dur[0]<dur[1]) {
                                if (dur[1]<=dur[0]+minSep) {
                                    thisOutcome = "FixedSep" + policy2
                                }
                                else if (dur[1]/dur[0]<1+percentSep) {
                                    thisOutcome = "PercentSep" + policy2
                                }
                                else {
                                    thisOutcome =  policy2
                                }
                            }
                            else {
                                thisOutcome = "Equal"
                            }

                        }
                    }

                }
                outcome[[dFreq,s]] = thisOutcome
            }
        }
        List finalOut = []
        outcome.each {
            finalOut += [[it.key[0], it.key[1], it.value]]
        }
        return finalOut
    }
    
    void dumpWinner(int gens,
        Simulation sims,
        List<List<Integer>> drugMuts, double thres,
        int epiType,
        List<Integer> fields,
        String policy1, String policy2,
        int minSep, double percentSep) {
        dump(reportWinner(gens, sims, drugMuts, thres, epiType,
                fields, policy1, policy2, minSep, percentSep))
    }

    int[] countWinner(int gens,
        Simulation sims,
        List<List<Integer>> drugMuts, double thres,
        int epiType,
        List<Integer> fields,
        String policy1, String policy2,
        int minSep, double percentSep) {
        int[] counts = new double[3]
        List winners = reportWinner(gens, sims, drugMuts, thres, epiType,
            fields, policy1, policy2, minSep, percentSep)
        for (elem in winners) {
            if (elem[2].startsWith(policy1)) {
                counts[0]++
            }
            else if (elem[2].startsWith(policy2)) {
                counts[1]++
            }
            else {
                counts[2]++
            }
        }
        return counts
    }

    
    
    //Type means if fixates sensitive, sensitive disapears
    //also reports last senstive frequency
    List reportType(
        Map<List, PolicyResult> results,
        List<List<Integer>> drugMuts, double thres,
        List<Integer> fields) {
        List unsortedResults = []
        List sortedResults = []
        results.each {
            int status = 0;
            /*
              status:
                 -1 - sensitive wins
                  0 - equilibrium at the middle
                  1 - sensitive loses
            */
            double lastSensFreq
            lastSensFreq = it.genFreqs[-1][0]
            if (lastSensFreq<0.01) status = 1
            else if (lastSensFreq>0.99) status = -1
            unsortedResults += [[it.key[0], it.key[1], it.key[2],
                status, lastSensFreq]]
        }
        sortedResults = sort(unsortedResults, fields)
        return sortedResults
        dump(sortedResults)
    }


    void dumpType(
        Map<List, PolicyResult> results,
        List<List<Integer>> drugMuts, double thres,
        List<Integer> fields) {
        dump(returnType(results, drugMuts, thres, fields))
        }
    
    void dump(List list) {
        list.each { tuple ->
            tuple.each {elem-> ps.print(elem + "\t")}
            ps.println()
        }
    }
}

