package malaria

public class TopSim {
    List<List<Integer>> drugMuts
    int numMuts
    List<Double> dFreqs = []
    List<Double> reducedDFreqs = []
    List<Double> resLevels = [0.0D]
    List<Double> Gs = []
    List<Double> Ss = []
    List<Double> reducedSs = []
    List<Double> TSs = []
    List<Double> reducedTSs = []
    String pol
    int GENS
    double THRESHOLD
    double START_FREQ
    boolean genGP
    int SAMPLE_SIZE
    boolean doTS
    int policy
    int numDrugs
    int locPerDrug
    String rotation
    boolean isSI
    boolean isACT
    int epiType
    int minInterval = 0

    boolean introduction
    double probIntro
    double percentReplaced

    Simulation sim
    String simType
    String simAbbrev
    String cmAbbrev

    Table table
    static Map<String,String> defaults = [
        "GENS"             : "1000",
        "THRESHOLD"        : "0.1", //0.3
        "START_FREQ"       : "0.003", //0.0003
        "genGP"            : "false",
        "SAMPLE_SIZE"      : "100",
        "doTS"             : "false",
        "policy"           : "1",
        "numDrugs"         : "2",
        "locPerDrug"       : "1",
        "rotation"         : "Fixed/10",
        "isSI"             : "false",
        "resLevels"        : "0.0D", //Check this in the future
        "Gs"               : "0.05D",
        "dFreqs"           : "0.3D",
        "workMults"        : "[1:0.75D, 2:0.25D]",
        "isACT"            : "false",
        "epiType"          : "0",
        "introduction"     : "false",
        "PROB_INTRO"       : "0.001",
        "PERCENT_REPLACED" : "0.001"
    ]

    String writeDir
    StatusPane pane
    ChartPane chartPane

    public TopSim(String writeDir) {
        this.writeDir = writeDir
        this.pane = null
        this.chartPane = null
    }

    public TopSim(String writeDir, StatusPane pane, ChartPane chartPane) {
        this.writeDir = writeDir
        this.pane = pane
        this.chartPane = chartPane
    }

    def evalClean = { String str ->
        return Eval.me(str.replace("\\",""))
    }

    List<Double> getRange(String range) {
        if (range.indexOf("/")==-1) {
            return [Double.parseDouble(range)]
        }
        else {
            double startV
            double endV
            double stepV
            List<String> toks = range.split("/")
            startV = Double.parseDouble(toks[0])
            endV = Double.parseDouble(toks[1])
            stepV = Double.parseDouble(toks[2])
            List<Double> res = []
            double currentV = startV
            while (currentV<=endV){
                res += currentV
                currentV += stepV

            }
            return res
        }
    }

    public static Properties getDefaults() {
        return new Properties(defaults)
    }

    public simulate(String directory, String fName) {
        Properties p = new Properties(defaults)
        try {
            p.load(new FileInputStream(directory + File.separator + fName))
        }
        catch (FileNotFoundException e) {
            //OK, will use defaults
        }
        p.store(new FileOutputStream(directory + File.separator + "Done.config"), "Done article")
        this.coreSimulate(p)
    }

    public coreSimulate(Properties p) {
        if (pane != null) pane.start()

        //Configuration starts here
        GENS = Integer.parseInt(p.getProperty("GENS"))
        THRESHOLD = Double.parseDouble(p.getProperty("THRESHOLD"))
        START_FREQ = Double.parseDouble(p.getProperty("START_FREQ"))
        genGP = Boolean.valueOf(p.getProperty("genGP"))
        SAMPLE_SIZE = Integer.parseInt(p.getProperty("SAMPLE_SIZE"))
        doTS = Boolean.valueOf(p.getProperty("doTS"))
        policy = Integer.parseInt(p.getProperty("policy"))
        numDrugs = Integer.parseInt(p.getProperty("numDrugs"))
        locPerDrug = Integer.parseInt(p.getProperty("locPerDrug"))
        isSI = Boolean.valueOf(p.getProperty("isSI"))
        isACT = Boolean.valueOf(p.getProperty("isACT"))
        //outputFile = p.getProperty("outputFile")
        //prodCharts = Boolean.valueOf(p.getProperty("prodCharts"))
        rotation = p.getProperty("rotation")
        epiType = Integer.parseInt(p.getProperty("epiType"))
        resLevels = getRange(p.getProperty("resLevels"))
        Gs = getRange(p.getProperty("Gs"))
        if (doTS) {
            TSs = Gs
            Ss  = [0D]
        }
        else {
            Ss  = Gs
            TSs = [0D]
        }

        dFreqs = getRange(p.getProperty("dFreqs"))

        HashMap<Integer,Double> workMults = []
        String strMult = p.getProperty("workMults")
        workMults = Eval.me(strMult)
        int rotValue
        rotValue = -1
        if (policy==0) {
            if (rotation.equals("Dyna")) {
                pol = "ThresRot"
                rotValue = -1
            }
            else {
                pol = "ThresRotFixed"
                rotValue = Integer.parseInt(rotation.split("/")[1])
            }
        }
        else if (policy==1) {
            pol = "MFT"
        }
        else {
            pol = "CT"
        }

        introduction = Boolean.valueOf(p.getProperty("introduction"))
        probIntro = Double.parseDouble(p.getProperty("PROB_INTRO"))
        percentReplaced = Double.parseDouble(p.getProperty("PERCENT_REPLACED"))

        //Configuration ends

        Persistence writer = new Persistence(writeDir)

        sim = new Simulation(GENS, writer, 0,
            START_FREQ, THRESHOLD, false, epiType,
            introduction, probIntro, percentReplaced)
        SimulationRunner sr = new SimulationRunner(sim,
            [workMults], resLevels, TSs, Ss, dFreqs, isSI)
        if (pane != null) sr.setPane(pane)
        if (chartPane != null) sr.setChartPane(chartPane)

        if (isACT) {
            sr.runACT(numDrugs, minInterval, policy, rotValue)
        }
        else {
            sr.runStandard(numDrugs, locPerDrug, minInterval, policy, rotValue)
        }

        if (pane != null) pane.generate()


        simType = numDrugs + ' drugs, ' + locPerDrug + ' locus/drug'
        simAbbrev = "" + numDrugs + "" + locPerDrug
        reducedSs = reduce(Ss, false)
        reducedTSs = reduce(TSs, false)
        reducedDFreqs = reduce(dFreqs, true)
        if (numDrugs==1) {
            if (locPerDrug==1) {
                drugMuts = [[0]]
                numMuts = 1
            }
            else {
                drugMuts = [[0, 1]]
                numMuts = 2
            }
        }
        else if (numDrugs==2) {
            if (locPerDrug==1) {
                drugMuts = [[0], [1]]
                numMuts = 2
            }
            else {
                if (isACT) {
                    drugMuts = [[0,1], [0,2]]
                    numMuts = 3
                }
                else {
                    drugMuts = [[0,1], [2,3]]
                    numMuts = 4
                }

            }
        }
        else if (numDrugs==3) {
            if (locPerDrug==1) {
                drugMuts = [[0], [1], [2]]
                    numMuts = 3
            }
            else {
                if (isACT) {
                    drugMuts = [[0,1], [0,2], [0,3]]
                    numMuts = 4
                }
                else {
                    drugMuts = [[0,1], [2,3], [4,5]]
                    numMuts = 6
                }
            }
        }
        else {
            throw new Exception("More than 3 drugs not supported")
        }

	
        Simulation reducedSim, cmSim
        cmSim = sim.getSimCM(workMults)
        cmAbbrev = workMults.toString()
        reducedSim = cmSim.getSimS(reducedSs)
        reducedSim = reducedSim.getSimTS(reducedTSs)
        reducedSim = reducedSim.getSimDrugFreq(reducedDFreqs)
        produceCharts(cmSim, cmSim)

        //reportCut(cmSim.getSimPrefix("MFT"), reducedSim.getSimPrefix("MFT"))
        //plotInfluence(cmSim)
        if (pane != null) pane.stop()
    }


    def produceCharts = { Simulation cmSim, Simulation reducedSim ->
        List<Double> GSs
        int[] m0 = [0]
        int[] m1 = [1]
        int[] m2 = [2]
        int[] m3 = [3]
        int[] m01 = [0,1]
        int[] m02 = [0,2] //ACT mixed
        int[] m23 = [2,3]
        int[][] sPos1 = [[], [0], [1], [0,1]]
        int[][] sPos2 = [[], [0,1], [2,3], [0,1,2,3], [1], [2], [1,2]]
        int[][] sACT = [[], [0,1], [0,2], [0,1,2], [1,2], [1], [2]]

        String fName = "multi" + simAbbrev + "-" + cmAbbrev

        int minSep
        double percentSep
        /* //WINNER PLOTING
        for (mSpS in [[0,0D],[1,0D],[10,0.05D]]) {
            minSep = mSpS[0]
            percentSep = mSpS[1]
            fName = "winner" + simAbbrev + "-" + minSep + "-" + percentSep + "-" +
                cmAbbrev
            chart = chartObj.plotWinner(
                GENS, THRESHOLD, epiType, (int[][])drugMuts.toArray(), cmSim,
                origPol, origPolName, comparePol, comparePolName,
                doTS, minSep, percentSep, fName + ".txt")
            out = new FileOutputStream(fName + ".png")
            ChartObj.saveChart(out, chart)
            out.close()
        }
        */

        if (doTS) {
            GSs = TSs
        }
        else {
            GSs = Ss
        }
        for (dFreq in dFreqs) {
            Simulation dSim = reducedSim.getSimDrugFreq(dFreq ,dFreq)
            for (gs in GSs) {
                Simulation dsSim
                if (doTS) {
                    dsSim = dSim.getSimTS(gs, gs)
                }
                else {
                    dsSim = dSim.getSimS(gs, gs)
                }
                if (dsSim.results.size()>0) {
                    Simulation polSim = dsSim.getSimPrefix(pol)
                    fName = pol + "-" + simAbbrev + ("+" + Math.round(gs*1000)/1000).replace(".","") + ("+" + Math.round(dFreq*1000)/1000).replace(".","")
                    NoChart.plotCase(
                        polSim.results.values().toArray()[0], drugMuts, epiType,
                        this.writeDir, fName + ".og")
                    if (genGP) {
                        NoChart.plotGenepop(
                            polSim.results.values().toArray()[0], numMuts, epiType,
                            this.writeDir, fName + ".txt", SAMPLE_SIZE)
                    }
                    NoChart.plotSwaps(
                        polSim.results.values().toArray()[0],
                        this.writeDir, fName + ".sw")
                    if (numDrugs == 1 & locPerDrug == 2) {
                        NoChart.plotLD(
                            polSim.results.values().toArray()[0],
                            this.writeDir, "LD" + fName + ".txt")
                    }
                }
                if (pane != null) pane.increment()
            }
        }

    }



    List<Double> reduce(List<Double> bigList, boolean isDrug) {
        List <Double> reducedList = []
        for (item in bigList) {
            if (item in ([0.02D, 0.06D, 0.99D] + ((0..9) *. div(10D)))) reducedList += item
            if (isDrug && item==0.99D) reducedList += item
            if ((!isDrug) && item==1D) reducedList += item
        }
        System.out.println("RL" + reducedList)
        return reducedList
    }


}