package malaria

import groovy.swing.SwingBuilder
import java.awt.BorderLayout as BL
import java.awt.GridLayout as GL
import java.awt.Toolkit
import javax.swing.BoxLayout
import javax.swing.JCheckBox
import javax.swing.JComboBox
import javax.swing.JDialog
import javax.swing.JFormattedTextField
import javax.swing.JFrame
import javax.swing.JLabel
import javax.swing.JOptionPane
import javax.swing.JPanel
import javax.swing.JTextField
import javax.swing.JRadioButton
import javax.swing.JFileChooser
import javax.swing.JMenuItem
import javax.swing.JSlider
import javax.swing.JButton
import java.text.DecimalFormat
import javax.swing.JMenu
import javax.swing.ImageIcon
import java.util.jar.JarInputStream
import java.util.zip.ZipEntry
import java.util.zip.ZipInputStream

/**
 *
 * @author tra
 */
class UI {
    String    currentProject
    String    directory
    JMenu     openMenu

    SwingBuilder swing = new SwingBuilder()
    JMenuItem saveProject
    JFrame    frame
    JPanel    inputPanel
    JButton   seqButton
    JComboBox numDrugs
    JComboBox drugType
    JComboBox locPerDrug
    JComboBox epistasis    //Epistasis will be a number 0..3
    JComboBox policy
    JComboBox penalty
    JComboBox infectivity
    JLabel    rotation
    JLabel    usage
    JLabel    fitness
    JLabel    logo
    JFormattedTextField generations
    JFormattedTextField threshold
    JCheckBox introduction
    JFormattedTextField probIntro
    JFormattedTextField percentReplaced
    JFormattedTextField startFreq
    JCheckBox genGP
    JFormattedTextField sampleSize

    Map<Integer, Double>    workMults

    StatusPane statusPane
    ChartPane chartPane

    public static void main(String[] args) {
        UI ui = new UI()
        ui.go()
    }

    public void readProps(Properties p) {
        generations.value = Integer.parseInt(p.getProperty("GENS", "100"))
        threshold.value = Double.parseDouble(p.getProperty("THRESHOLD", "0.1"))
        startFreq.value = Double.parseDouble(p.getProperty("START_FREQ", "0.001"))
        if (Boolean.valueOf(p.getProperty("genGP", "false"))) {
            genGP.setSelected(true)
            sampleSize.enabled = true
        }
        else {
            genGP.setSelected(false)
            sampleSize.enabled = false
        }
        sampleSize.value = Integer.parseInt(p.getProperty("SAMPLE_SIZE", "100"))
        if (Boolean.valueOf(p.getProperty("doTS", "false"))) {
            penalty.selectedIndex = 1
        }
        else {
            penalty.selectedIndex = 0
        }
        numDrugs.selectedIndex = Integer.parseInt(p.getProperty("numDrugs", "2")) - 1
        locPerDrug.selectedIndex = Integer.parseInt(p.getProperty("locPerDrug", "2")) - 1
        if (Boolean.valueOf(p.getProperty("isSI", "false"))) {
            infectivity.selectedIndex = 1
        }
        else {
            infectivity.selectedIndex = 0
        }
        if (Boolean.valueOf(p.getProperty("isACT", "false"))) {
            drugType.selectedIndex = 1
        }
        else {
            drugType.selectedIndex = 0
        }
        epistasis.selectedIndex = Integer.parseInt(p.getProperty("epiType", "0"))
        policy.selectedIndex = Integer.parseInt(p.getProperty("policy", "1"))
        rotation.text = p.getProperty("rotation", "Fixed/10")
        usage.text = p.getProperty("dFreqs", "0.3")
        fitness.text = p.getProperty("Gs","0.05")
        workMults = Eval.me(p.getProperty("workMults", "[2 : 1.0D]").replace("\\",""))

        //prodCharts = Boolean.valueOf(p.getProperty("prodCharts"))
        //residual druglevels

        if (Boolean.valueOf(p.getProperty("introduction", "false"))) {
            introduction.setSelected(true)
            probIntro.enabled = true
            percentReplaced.enabled = true
        }
        else {
            introduction.setSelected(false)
            probIntro.enabled = false
            percentReplaced.enabled = false
        }
        probIntro.value = Double.parseDouble(p.getProperty("PROB_INTRO", "0.01"))
        percentReplaced.value = Double.parseDouble(p.getProperty("PERCENT_REPLACED", "0.001"))

    }


    String getMapIntDoubleAsString(Map<Integer,Double> map) {
        String s = "["
        boolean first = true
        map.each {
            if (it.value != 0D) {
                if (first) {
                    first = false
                }
                else {
                    s += ", "
                }
                s += it.key.toString()+ ": " + it.value.toString() + "D"
            }
        }
        return s + "]"
    }

    public void writeProps(File f) {
        Properties p = new Properties()

        p.setProperty("GENS", generations.value.toString())
        p.setProperty("THRESHOLD", threshold.value.toString())
        p.setProperty("START_FREQ", startFreq.value.toString())
        if (genGP.isSelected()) {
            p.setProperty("genGP", "true")
        }
        else {
            p.setProperty("genGP", "false")
        }
        p.setProperty("SAMPLE_SIZE", sampleSize.value.toString())
        if (penalty.selectedIndex == 0) {
            p.setProperty("doTS", "false")
        }
        else {
            p.setProperty("doTS", "true")
        }
        p.setProperty("numDrugs", (numDrugs.selectedIndex+1).toString())
        p.setProperty("locPerDrug", (locPerDrug.selectedIndex+1).toString())
        if (infectivity.selectedIndex == 0) {
            p.setProperty("isSI", "false")
        }
        else {
            p.setProperty("isSI", "true")
        }
        if (drugType.selectedIndex == 0) {
            p.setProperty("isACT", "false")
        }
        else {
            p.setProperty("isACT", "true")
        }
        p.setProperty("epiType", epistasis.selectedIndex.toString())
        p.setProperty("policy", policy.selectedIndex.toString())
        p.setProperty("rotation", rotation.text)
        p.setProperty("dFreqs", usage.text)
        p.setProperty("Gs", fitness.text)
        p.setProperty("workMults", getMapIntDoubleAsString(workMults))

        //prodCharts = Boolean.valueOf(p.getProperty("prodCharts"))
        //residual druglevels

        if (introduction.isSelected()) {
            p.setProperty("introduction", "true")
        }
        else {
            p.setProperty("introduction", "false")
        }
        p.setProperty("PROB_INTRO", probIntro.text)
        p.setProperty("PERCENT_REPLACED", percentReplaced.text)

        p.store(new FileOutputStream(f), "ogaraK UI")
    }

    public void askMOI() {
        JDialog MOIDialog
        Double v1, v2, v3, v4, v5, v6, v7
        JFormattedTextField m1, m2, m3, m4, m5, m6, m7
        MOIDialog = swing.dialog(owner: frame, modal: true,
            title: "Rotation policy", size:[400, 300]) {
            borderLayout()
            panel(constraints:BL.CENTER) {
                bGroup = buttonGroup()
                gridLayout(columns: 2, rows: 7)
                label("Selfing")
                m1 = formattedTextField(value: workMults[1] ?: 0.0)
                label("2")
                m2 = formattedTextField(value: workMults[2] ?: 0.0)
                label("3")
                m3 = formattedTextField(value: workMults[3] ?: 0.0)
                label("4")
                m4 = formattedTextField(value: workMults[4] ?: 0.0)
                label("5")
                m5 = formattedTextField(value: workMults[5] ?: 0.0)
                label("6")
                m6 = formattedTextField(value: workMults[6] ?: 0.0)
                label("7")
                m7 = formattedTextField(value: workMults[7] ?: 0.0)
            }
            panel(constraints:BL.SOUTH) {
                button(text:"OK",
                    actionPerformed: {
                        double total =  m1.value + m2.value + m3.value + m4.value + m5.value + m6.value + m7.value
                        if (total != 1) {
                            JOptionPane.showMessageDialog(MOIDialog,"Total has to sum 1.0 (currently ${total})")
                        }
                        else {
                            workMults = [:]
                            if (m1.value != 0.0D) workMults[1] = m1.value
                            if (m2.value != 0.0D) workMults[2] = m2.value
                            if (m3.value != 0.0D) workMults[3] = m3.value
                            if (m4.value != 0.0D) workMults[4] = m4.value
                            if (m5.value != 0.0D) workMults[5] = m5.value
                            if (m6.value != 0.0D) workMults[6] = m6.value
                            if (m7.value != 0.0D) workMults[7] = m7.value
                            MOIDialog.dispose()
                        }
                    }
                )
                button(text:"Cancel",
                    actionPerformed: {
                        MOIDialog.dispose()
                    }
                )
            }
        }
        MOIDialog.pack()
        MOIDialog.show()
    }

    public void askRotation() {
        JDialog rotationDialog
        JRadioButton dynaButton
        int value
        boolean isFixed
        JSlider fixedSlider
        //JSlider dynaSlider

        if (rotation.getText().startsWith("Fixed")) {
            isFixed = true
            value = Integer.parseInt(rotation.getText().split("/")[1])
        }
        else {
            isFixed = false
        }

        rotationDialog = swing.dialog(owner: frame, modal: true,
            title: "Rotation policy", size:[480, 300]) {
            borderLayout()
            panel(constraints:BL.CENTER) {
                bGroup = buttonGroup()
                gridLayout(columns: 1, rows: 3)
                radioButton(text: "Fixed", buttonGroup: bGroup,
                    
                    selected: isFixed, actionPerformed: {
                        if (isFixed) return
                        isFixed = true
                        fixedSlider.enabled = true
                        //dynaSlider.enabled = false
                    }
                )
                panel() {
                    boxLayout(axis: BoxLayout.Y_AXIS)
                    label(text: "Generations",
                        horizontalAlignment: JLabel.CENTER)
                    fixedSlider = slider(enabled: isFixed, id: "fSlider",
                        minorTickSpacing: 5, majorTickSpacing: 10,
                        paintTicks: true, paintLabels: true,
                        minimum:10, maximum:100,
                        value: value?: 12
                    )
                    label(text: bind(source: fSlider, sourceProperty: "value"))
                }
                dynaButton = radioButton(text: "Dynamic", buttonGroup: bGroup,
                    selected: !isFixed, actionPerformed: {
                        if (!isFixed) return
                        isFixed = false
                        fixedSlider.enabled = false
                        //dynaSlider.enabled = true
                    }
                )
                /* panel() {
                boxLayout(axis: BoxLayout.Y_AXIS)
                label(text: "Threshold")
                dynaSlider = slider(enabled: !isFixed, id: "dSlider",
                minorTickSpacing: 5, majorTickSpacing: 20,
                paintTicks: true, paintLabels: true,
                value: value
                )
                label(text: bind(source: dSlider, sourceProperty: "value"))
                } */
            }
            panel(constraints:BL.SOUTH) {
                button(text:"OK",
                    actionPerformed: {
                        if (dynaButton.selected) {
                            rotation.text = "Dyna"// + "/" + dynaSlider.value
                        }
                        else {
                            rotation.text = "Fixed/" + fixedSlider.value
                        }
                        rotationDialog.dispose()
                        //frame.setExtendedState(frame.getExtendedState()|JFrame./MAXIMIZED_BOTH)
                        //frame.pack()
                        //frame.show()
                    }
                )
                button(text:"Cancel",
                    actionPerformed: {
                        rotationDialog.dispose()
                    }
                )

            }
        }
        //rotationDialog.pack()
        rotationDialog.show()

    }

    public void askRange(JLabel workLabel, String myText) {
        JDialog rangeDialog
        JRadioButton rangeButton
        double singleV
        JFormattedTextField single
        double startV
        JFormattedTextField start
        double stopV
        JFormattedTextField stop
        double stepV
        JFormattedTextField step
        boolean isRange
        if (workLabel.getText().indexOf("/")>-1) {
            String[] parts = workLabel.getText().split("/")
            isRange = true
            singleV = 0.1
            startV  = Double.parseDouble(parts[0])
            stopV   = Double.parseDouble(parts[1])
            stepV   = Double.parseDouble(parts[2])
        }
        else {
            isRange = false
            singleV = Double.parseDouble(workLabel.getText())
            startV  = 0.0
            stopV   = 1.0
            stepV   = 0.2
        }
        rangeDialog = swing.dialog(owner: frame, modal: true,
            title: myText, size:[300,300]) {
            borderLayout()
            panel(constraints:BL.CENTER) {
                bGroup = buttonGroup()
                gridLayout(columns:2, rows:6)
                radioButton(text: "Single value", buttonGroup: bGroup,
                    selected: !isRange, actionPerformed: {
                        single.enabled = true
                        start.enabled = false
                        stop.enabled = false
                        step.enabled = false
                    })
                label(text: "")
                label(text: myText)
                single = formattedTextField(enabled: !isRange, value: singleV)
                rangeButton = radioButton(text: "Range", buttonGroup: bGroup,
                    selected: isRange, actionPerformed: {
                        single.enabled = false
                        start.enabled = true
                        stop.enabled = true
                        step.enabled = true
                    }
                )
                label(text: "")
                label(text: "Start")
                start = formattedTextField(enabled: isRange, value: startV)
                label(text: "Stop")
                stop = formattedTextField(enabled: isRange, value: stopV)
                label(text: "Step")
                step = formattedTextField(enabled: isRange, value: stepV)
            }
            panel(constraints:BL.SOUTH) {
                button(text:"OK",
                    actionPerformed: {
                        if (rangeButton.selected) {
                            if (start.value<0.0 || start.value>1.0) {
                                JOptionPane.showMessageDialog(frame, "Start has to be between 0 and 1");
                            }
                            else {
                                if ((stop.value<0.0 || stop.value>1.0) || stop.value<start.value) {
                                    JOptionPane.showMessageDialog(frame, "Stop has to be between 0 and 1 and BIGGER THAN Start");
                                }
                                else {
                                    if (step.value<=0.0 || step.value>1.0) {
                                        JOptionPane.showMessageDialog(frame, "Step has to be between 0 and 1");
                                    }
                                    else {
                                        workLabel.text = start.value.toString() + "/" + stop.value.toString() + "/" +step.value.toString()
                                        rangeDialog.dispose()
                                        //frame.pack()
                                        //frame.show()
                                    }
                                }
                            }

                        }
                        else {
                            if (single.value<0.0 || single.value>1.0) {
                                JOptionPane.showMessageDialog(frame, "Value has to be between 0 and 1");
                            }
                            else {
                                workLabel.text = single.value.toString()
                                rangeDialog.dispose()
                                //frame.setExtendedState(frame.getExtendedState()|JFrame.MAXIMIZED_BOTH)
                                //frame.pack()
                                //frame.show()
                            }
                        }
                    }
                )
                button(text:"Cancel",
                    actionPerformed: {
                        rangeDialog.dispose()
                    }
                )

            }
        }
        rangeDialog.show()

    }

    public void loadFile() {
        Properties props = new Properties()
        props.load(new FileInputStream(new File(directory + File.separator +
                    currentProject + File.separator + "config")))
        readProps(props);
        frame.title = "ogaraK - " + currentProject
        //currentDirectory = f.getParent()
        //currentProject = f.getName()
        saveProject.enabled = true
    }


    public void saveFile(File f) {
        writeProps(f)
        saveProject.enabled = true
    }

    public void saveFile() {
        //println currentDirectory
        saveFile(new File (directory + File.separator + currentProject + File.separator + "config"))
    }

    void copyFile(InputStream is, String name, String destDir) {
        FileOutputStream fos = new FileOutputStream(destDir + File.separator + name)
        int avl = is.available()
        byte[] content = new byte[avl]
        is.read(content, 0, avl)
        fos.write(content)
        fos.close()
    }

    void copyFile(ZipInputStream zis, int avl, String name, String destDir) {
        FileOutputStream fos = new FileOutputStream(destDir + File.separator + name)
        byte[] content = new byte[avl]
        zis.read(content, 0, avl)
        fos.write(content)
        fos.close()
    }


    void installContent(String directory) {
        ClassLoader cl = this.getClass().getClassLoader()
        copyFile(cl.getResourceAsStream("tiny.png"), "tiny.png", directory)
        ZipInputStream zis = new ZipInputStream(cl.getResourceAsStream("examples.zip"))
        ZipEntry ze
        while ((ze = zis.getNextEntry()) != null){
            if (ze.isDirectory()) {
                (new File(directory + File.separator + ze.getName())).mkdir()
            }
            else {
                String dir = ze.getName().split("/")[0]
                String fName = ze.getName().split("/")[1]
                copyFile(zis, (int)ze.getSize(), fName, directory + File.separator + dir)
            }
        }

    }

    protected ImageIcon createImageIcon(String resource) {
        java.net.URL imgURL = this.getClass().getClassLoader().getResource(resource)
        if (imgURL != null) {
            return new ImageIcon(imgURL)
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }


    public void go() {
        currentProject = null
        directory = System.getProperty("user.home") + File.separator + "ogarak"
        if ((new File(directory)).mkdir()) {
            installContent(directory)
        }
        workMults = [2: 1.0D]
        statusPane = new StatusPane(this)
        chartPane = new ChartPane()

        frame = swing.frame(title: 'ogaraK') { //}, size:[300, 640]) {
            borderLayout()
            menuBar() {
                menu(text: "Projects", mnemonic: 'F') {
                    openMenu = menu (text: "Open", mnemonic: "O") {
                        String[] list = (new File(directory)).list()
                        list.sort()
                        for (entry in list) {
                            if ((new File(directory + File.separator + entry +
                                        File.separator + "config")).exists()) {
                                menuItem(text: entry, actionPerformed: { evt ->
                                        currentProject = evt.source.text
                                        loadFile()
                                        saveProject.enabled = true
                                    }
                                )
                            }
                        }
                    }
                    menuItem (text: "Save as...", mnemonic: "A", actionPerformed: {
                            dialog = swing.dialog(owner: frame,
                                title: "New Project") {
                                gridLayout(columns:2, rows:2)
                                label(text:"Name",
                                    horizontalAlignment: JLabel.CENTER)
                                newName = textField(columns:15)
                                button(text:'OK', actionPerformed: {
                                        name = newName.text
                                        if (name.contains(File.separator)) {
                                            d2 = JOptionPane.showMessageDialog(
                                                dialog,
                                                "Name cannot contain character: " + File.separator)
                                            return
                                        }
                                        myDir = directory + File.separator + name
                                        if (!(new File(myDir).mkdir())) {
                                            d2 = JOptionPane.showMessageDialog(
                                                dialog,
                                                "Could not create project directory")
                                            return
                                        }
                                        currentProject = name
                                        saveProject.enabled = true
                                        saveFile()
                                        frame.title = "ogaraK - " + currentProject
                                        openMenu.add(swing.menuItem(text: name, actionPerformed: {
                                                    currentProject = name
                                                    loadFile()
                                                    saveProject.enabled = true
                                                }
                                            ))
                                        dialog.dispose()
                                    }
                                )
                                button(text:'Cancel', actionPerformed: {
                                        dialog.dispose()
                                    }
                                )
                            }
                            dialog.pack()
                            dialog.show()
                        }
                    )

                    /*
                    menuItem(text: "Open Project", mnemonic: 'O', actionPerformed: {
                    JFileChooser chooser = new JFileChooser();
                    int retVal = chooser.showOpenDialog(frame)
                    if (retVal == JFileChooser.APPROVE_OPTION) {
                    loadFile(chooser.getSelectedFile())
                    }
                    }
                    )*/
                    saveProject = menuItem(text: "Save",
                        mnemonic: 'S', enabled: false, actionPerformed: {
                            saveFile()
                        }
                    )
                    /*
                    menuItem(text: "Save Project As...", mnemonic: 'A',
                    actionPerformed: {
                    JFileChooser chooser = new JFileChooser();
                    int retVal = chooser.showSaveDialog(frame)
                    if (retVal == JFileChooser.APPROVE_OPTION) {
                    saveFile(chooser.getSelectedFile())
                    currentDirectory = chooser.getSelectedFile().getParent()
                    currentProject = chooser.getSelectedFile().getName()
                    }
                    }
                    )
                     */
                    separator()
                    menuItem(text: "Exit", mnemonic: 'X',
                        actionPerformed: {dispose() })
                }
                menu(text: "Help", mnemonic: "H") {
                    menuItem(text: "Data directory", mnemonic: 'D',
                        actionPerformed: {
                            JOptionPane.showMessageDialog(frame,
                                "Project data can be found on " + directory)
                        }
                    )
                    menuItem(text: "General help", mnemonic: 'H',
                        actionPerformed: {
                            JOptionPane.showMessageDialog(frame,
                                "For help please go to http://popgen.eu/soft/ogaraK")

                        }
                    )
                }
            }
            panel(constraints: BL.WEST) {
                label(text:" ")
            }
            panel(constraints: BL.CENTER) {
                gridLayout(columns:2, rows:1)
                inputPanel = panel(enabled: false) {
                    gridLayout(columns: 2, rows: 19)
                    label(text: "Number of drugs")
                    numDrugs = comboBox(items: [1, 2, 3],
                        actionPerformed: {
                            if (numDrugs.selectedIndex == 0) {
                                policy.selectedIndex = 1
                                policy.enabled = false
                            }
                            else {
                                policy.enabled = true
                            }
                        }
                    )
                    label(text: "Drug type")
                    drugType = comboBox(items: ["Standard", "Artemisinin"],
                        actionPerformed: {
                            if (drugType.selectedIndex == 0) {
                                locPerDrug.enabled = true
                            }
                            else {
                                locPerDrug.selectedIndex = 1
                                locPerDrug.enabled = false
                            }
                        }
                    )
                    label(text: "Loci per drug")
                    locPerDrug = comboBox(items: [1, 2])
                    label(text: "Epistasis")
                    epistasis = comboBox(items: ["Full", "Duplication", "Full+Dup", "Full+Assym"])
                    label(text: "Policy")
                    policy = comboBox(items: ["Sequential", "Multiple First Line", "Combination"],
                        selectedIndex: 1,
                        enabled: false,
                        actionPerformed: {
                            if (policy.selectedIndex==0) {
                                rotation.enabled = true
                                seqButton.enabled = true
                            }
                            else {
                                rotation.enabled = false
                                seqButton.enabled = false
                            }
                        }
                    )
                    label(text: "Rotation (Sequencial)")
                    panel() {
                        rotation = label(text: "Fixed/10")
                        seqButton = button(text:'Change',
                            actionPerformed: {
                                askRotation()
                            },
                            enabled: policy.selectedIndex == 0
                        )
                    }
                    label(text: "Infectivity")
                    infectivity = comboBox(items: ["Competitive release", "Independent transmission"])
                    label(text: "Fitness penalty")
                    penalty = comboBox(items: ["Erythrocytic", "Exo-erythrocytic"])
                    label(text: "Penalty value")
                    panel() {
                        fitness = label(text: "0.05")
                        button(text: 'Change',
                            actionPerformed: {
                                askRange(fitness, "Fitness penalty")
                            }
                        )
                    }
                    label(text: "Drug usage")
                    panel() {
                        usage = label(text: "0.3")
                        button(text:'Change',
                            actionPerformed: {
                                askRange(usage, "Drug usage")
                            }
                        )
                    }
                    label(text: "MOI")
                    button(text:'View/Change',
                        actionPerformed: {
                            askMOI()
                        }
                    )
                    label(text: "Resistance Introduction")
                    introduction = checkBox(actionPerformed : {
                            if (introduction.isSelected()) {
                                probIntro.enabled = true
                                percentReplaced.enabled = true
                            }
                            else {
                                probIntro.enabled = false
                                percentReplaced.enabled = false
                            }
                        }
                    )
                    label(text: "Prob. Introduction")
                    probIntro = formattedTextField(
                        format: new DecimalFormat("0.000000"),
                        value: new Double(0.01),
                        enabled: false
                    )
                    label(text: "Percent replaced")
                    percentReplaced = formattedTextField(
                        format: new DecimalFormat("0.0000"),
                        enabled: false,
                        value: new Double(0.001)
                    )
                    label(text: "Max generations")
                    generations = formattedTextField(value: new Integer(100))
                    label(text: "Threshold")
                    threshold = formattedTextField(format: new DecimalFormat("0.00"), value: new Double(0.1))
                    label(text: "Initial frequency")
                    startFreq = formattedTextField(format: new DecimalFormat("0.0000"), value: new Double(0.001))
                    label(text: "Generate genepop?")
                    genGP = checkBox(actionPerformed : {
                            if (genGP.isSelected()) {
                                sampleSize.enabled = true
                            }
                            else {
                                sampleSize.enabled = false
                            }
                        }
                    )
                    label(text: "Sample size (Genepop)")
                    sampleSize = formattedTextField(value: new Integer(100),
                        enabled: false)
                    //minimum interval
                    //residual levels
                    //producing charts goes elsewhere....
                }
                /*
                textlabel = label(text:"Click the button!", constraints: BL.NORTH)
                button(text:'Click Me',
                actionPerformed: {
                count++;
                textlabel.text = "Clicked ${count} time(s).";
                println "clicked"
                },
                constraints:BL.SOUTH
                )
                 */
            }
            panel(constraints: BL.EAST) {
                borderLayout()
                p = panel(constraints: BL.CENTER) {
                    logo = label(icon: createImageIcon("tiny.png"))
                }
                p.add(statusPane.getPane())
                p2 = panel(constraints: BL.SOUTH) {
                }
                p2.add(chartPane.getPane())
            }
           
            panel(constraints: BL.SOUTH) {
                button(text: "Recompute cost", actionPerformed: {
                        statusPane.recompute()
                    }
                )
                button(text: "Run Simulations", actionPerformed: {
                        if (currentProject == null) {
                            JOptionPane.showMessageDialog(frame, "You need to save first (File->Save As...)")
                            return
                        }
                        saveFile()
                        statusPane.recompute()
                        TopSim sim = new TopSim(directory + File.separator + currentProject, statusPane, chartPane)
                        sim.simulate(directory + File.separator + currentProject, "config" )
                    }
                )
            }
        }
        frame.pack()
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE)
        //frame.setExtendedState(frame.getExtendedState()|JFrame.MAXIMIZED_BOTH)
        frame.setVisible(true)
        statusPane.recompute()
    }
}