/*
 * Tests TopSim.
 *
 * This is more a way of integration testing that unit testing.
 */

package malaria;

import java.io.File;
import java.io.IOException;
//import junit.framework.TestCase;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import java.util.Properties;


/**
 *
 * @author tiago
 */
public class TopSimTest {
    static File dir = new File("testd");
    Properties config;
    TopSim topSim;

    @BeforeClass public static void setUpClass() throws Exception {
        if (!dir.mkdir()) {
            throw new IOException("Cannot create test directory");
        }
    }

    @AfterClass public static void tearDownClass() throws Exception {
        /*
        if (!deleteDirectory(dir)) {
            throw new IOException("Could not clean up test directory, please clean manually");
        }
         * */
        System.out.println("Re-enact tear down");
    }

    public static boolean deleteDirectory(File path) {
        if( path.exists() ) {
            File[] files = path.listFiles();
            for(int i=0; i<files.length; i++) {
                if(files[i].isDirectory()) {
                    deleteDirectory(files[i]);
                }
                else {
                    files[i].delete();
                }
            }
        }
        return( path.delete() );
    }

    void setBaseConfig() {
        config = new Properties(TopSim.getDefaults());
        config.put("GENS"             , "100");
        config.put("THRESHOLD"        , "0.1"); //0.3
        config.put("START_FREQ"       , "0.003"); //0.0003
        config.put("SAMPLE_SIZE"      , "100");
        config.put("doTS"             , "false");
        config.put("policy"           , "1");
        config.put("numDrugs"         , "2");
        config.put("locPerDrug"       , "1");
        config.put("rotation"         , "Fixed/10");
        config.put("isSI"             , "false");
        config.put("resLevels"        , "0.0D"); //Check this in the future
        config.put("Gs"               , "0.05D");
        config.put("dFreqs"           , "0.3D");
        config.put("workMults"        , "[1:0.75D, 2:0.25D]");
        config.put("isACT"            , "false");
        config.put("epiType"          , "0");
        config.put("introduction"     , "false");
        config.put("PROB_INTRO"       , "0.001");
        config.put("PERCENT_REPLACED" , "0.001");

    }

    @Before public void setUp() {
        topSim = new TopSim(dir.getAbsolutePath());
        setBaseConfig();
    }

    @After public void tearDown() {
    }

    @Test public void testSimpleStartup() {
        topSim.coreSimulate(config);
        OGParser parser =
                new OGParser(dir.getAbsolutePath() + dir.separator + "MFT-21+005+03.og");
        assertEquals(parser.getNumClones(), 4);
        assertEquals(parser.getNumGens(), 100);
        assertTrue(parser.getGenClone(50, 3) > 0.5);
        for (File f : dir.listFiles()) {
          System.out.println(f.getAbsoluteFile());
        }
    }



}