import sys
import pylab
import os
import re

if len(sys.argv) != 4:
    print "Arguments: %s <projDir> <epi> <MOI>" % (sys.argv[0])

projDir = sys.argv[1]
epi = sys.argv[2]
MOI = int(sys.argv[3])

maxDur = 200

def readFreqs(f):
    l = f.readline()
    freqs = {}
    while l!="":
        hapl, freq = l.rstrip().split()
        hapl = int(hapl)
        freq = float(freq)
        freqs.setdefault(hapl, []).append(freq)
        l = f.readline()
    f.close()
    return freqs, max(freqs.keys())

dirTpl = os.sep.join(["%s","2%s%s%d"])
fnameTpl = os.sep.join([dirTpl, "%s-22+%s+%s.og"])

pMFT = set()
pRot = set()

myRe = re.compile(".*\+([0-9]+)\+([0-9]+)\.og$")
for pol in ["Rot", "MFT"]:
    if pol=="Rot":
        mySet = pRot
    else:
        mySet = pMFT
    cases = os.listdir(dirTpl % (projDir, pol, epi, MOI))
    for case in cases:
        match = myRe.match(case)
        if match:
            mySet.add((match.group(1), match.group(2)))
intersection = pMFT.intersection(pRot)

def getDuration(sens):
    for i in range(len(sens)-3):
        limit = 1 - 0.1
        if sens[i]<limit and sens[i+1]<limit and sens[i+2]<limit:
            return i
    return maxDur

rot = {}
mft = {}
cnt = 0
for params in intersection:
    s = params[0]
    dFreq = params[1]
    rotName = fnameTpl %(projDir, "Rot", epi, MOI, "ThresRot", s, dFreq)
    mftName = fnameTpl %(projDir, "MFT", epi, MOI, "MFT", s, dFreq)
    rotFreqs, maxHaplo = readFreqs(open(rotName))
    mftFreqs, maxHaplo = readFreqs(open(mftName))
    
    s, dFreq = float(s[0]+"."+s[1:]), float(dFreq[0]+"."+dFreq[1:])
    if s>0.5: continue
    durRot = getDuration(rotFreqs[0])
    durMFT = getDuration(mftFreqs[0])
    #print "s"
    #print rotFreqs[0][:10]
    #print mftFreqs[0][:10]
    if durRot != durMFT: print s, dFreq
    if durRot<maxDur and durMFT<maxDur:
        mft.setdefault(s, []).append(durMFT)
        rot.setdefault(s, []).append(durRot)

    cnt += 1
    if cnt % 100 == 0: print cnt
    if cnt == 400: break

rotX = rot.keys()
rotX.sort()
rotY = []
for myRot in rotX:
    rotY.append(rot[myRot][len(rot[myRot])/2])

mftX = mft.keys()
mftX.sort()
mftY = []
for myMft in mftX:
    mftY.append(mft[myMft][len(mft[myMft])/2])

pylab.plot(mftX, mftY, label = "MFT")
pylab.plot(rotX, rotY, label = "Rot")


pylab.legend()
pylab.show()
